/*
 * AuthenticationProcess.java
 *
 * Created on January 7, 2003, 5:30 PM
 */

package examples.instantmessaging;

import gov.nist.javax.sip.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;
import examples.instantmessaging.presence.* ;
import examples.instantmessaging.authentication.* ;
/**
 *
 * @author  olivier deruelle
 */
public class AuthenticationProcess {
    
    private Vector usersTagList;
    private IMUserAgent imUA;
    
    /** Creates a new instance of AuthenticationProcess */
    public AuthenticationProcess(IMUserAgent imUA,Vector usersTagList) {
        this.imUA=imUA;
        
        if (usersTagList!=null) this.usersTagList=usersTagList;
        else this.usersTagList=new Vector();
    }
    
    public boolean hasLoginInformations(String realmParameter) {
        for(int i=0;i<usersTagList.size();i++) {
            UserTag userTag=(UserTag)usersTagList.elementAt(i);
            String realm=userTag.getUserRealm();
            if (realm!=null && realm.trim().equals(realmParameter) ) {
                return true;
            }
        }
        return false;
    }
    
    public String getUserName(String realmParameter) {
       for ( int i=0;i<usersTagList.size();i++) {
            UserTag userTag=(UserTag)usersTagList.elementAt(i);
            String realm=userTag.getUserRealm();
            if (realm!=null && realm.trim().equals(realmParameter) ) {
                return userTag.getUserName();
            }
       }
       return null;
    }
    
    public String getPassword(String realmParameter) {
        for(int i=0;i<usersTagList.size();i++) {
            UserTag userTag=(UserTag)usersTagList.elementAt(i);
            String realm=userTag.getUserRealm();
            if (realm!=null && realm.trim().equals(realmParameter) ) {
                return userTag.getUserPassword();
            }
        }
        return null;
    }
    
     public void addUser(String userName,String password,String realm) {
        UserTag userTag=new UserTag();
        userTag.setUserName(userName);
        userTag.setUserRealm(realm);
        userTag.setUserPassword(password);
        
        usersTagList.addElement(userTag);
    }
    
    public ProxyAuthorizationHeader getProxyAuthorizationHeader(Response response) {
        try {
            
            // Proxy-Authorization header:
            ProxyAuthenticateHeader authenticateHeader=(ProxyAuthenticateHeader)
            response.getHeader(
            ProxyAuthenticateHeader.NAME);
            CSeqHeader cseqHeader=(CSeqHeader)response.getHeader(CSeqHeader.NAME);
            
            String nonce=authenticateHeader.getNonce();
            String realm=authenticateHeader.getRealm();
            if (realm==null) {
                DebugIM.println("AuthenticationProcess, getProxyAuthorizationHeader(),"+
                " ERROR: the realm is not part of the 407 response!");
                return null;
            }
            String cnonce=authenticateHeader.getParameter("cnonce");
            String uri="sip:"+imUA.getRegistrarAddress()+":"+imUA.getRegistrarPort();
            String method=cseqHeader.getMethod();
            String userName=null;
            String password=null;
            
            if ( hasLoginInformations(realm) ) {
                // We can send the stored informations:
                userName=getUserName(realm);
                password=getPassword(realm);
            }
            else {
                // We have to ask the user:
                InstantMessagingGUI imGUI=imUA.getInstantMessagingGUI();
                AuthenticationDialog authenticationDialog=new AuthenticationDialog(imGUI,realm);
            
                if ( authenticationDialog.isStop()) return null;
                userName=authenticationDialog.getUserName();
                password=authenticationDialog.getPassword();
                
                // Let's store those informations:
                addUser(userName,password,realm);
            }
                
            HeaderFactory headerFactory=imUA.getHeaderFactory();
           
            DigestClientAuthenticationMethod digest=new DigestClientAuthenticationMethod();
            digest.initialize(realm,userName,uri,nonce,password,method,cnonce,"MD5");
            
            ProxyAuthorizationHeader header=headerFactory.createProxyAuthorizationHeader("Digest");
            header.setParameter("username",userName);
            header.setParameter("realm",realm);
            header.setParameter("uri",uri);
            header.setParameter("algorithm","MD5");
            header.setParameter("opaque","");
            header.setParameter("nonce",nonce);
            header.setParameter("response",digest.generateResponse());
            
            return header;
            
        }
        catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
}
