/*
 * IMMessageProcessing.java
 *
 * Created on September 26, 2002, 12:04 AM
 */

package examples.instantmessaging.presence;

import gov.nist.javax.sip.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;
import examples.instantmessaging.*;
/**
 *
 * @author  olivier
 * @version 1.0
 */
public class IMMessageProcessing {
    
    private IMUserAgent imUA;
    private int callIdCounter;
    private int cseq;
    
    
    /** Creates new IMMessageProcessing */
    public IMMessageProcessing( IMUserAgent imUA) {
        this.imUA=imUA;
        callIdCounter=0;
        cseq=0;
    }
    
    public Vector getRoutes(Message message) {
        try{
            MessageFactory messageFactory=imUA.getMessageFactory();
            SipProvider sipProvider=imUA.getSipProvider();
            HeaderFactory headerFactory=imUA.getHeaderFactory();
            AddressFactory addressFactory=imUA.getAddressFactory();
            ListIterator recordRoutes= message.getHeaders(RecordRouteHeader.NAME);
            if (recordRoutes != null && recordRoutes.hasNext() ) {
                
                Vector routes=new Vector();
                
                // we skip the first one
                RecordRouteHeader skip = (RecordRouteHeader)recordRoutes.next();
                while( recordRoutes.hasNext() ) {
                    RecordRouteHeader recordRouteHeader = (RecordRouteHeader)recordRoutes.next();
                    Address rrAddress=recordRouteHeader.getAddress();
                    
                    RouteHeader routeHeader=headerFactory.createRouteHeader(rrAddress);
                    routes.addElement(routeHeader);
                }
                
                // We have to add the peer:
                ListIterator viaHeaders=message.getHeaders(ViaHeader.NAME);
                ViaHeader viaHeader=null;
                while( viaHeaders.hasNext() ) {
                    viaHeader=(ViaHeader)viaHeaders.next();
                }
                String host=viaHeader.getHost();
                int port=viaHeader.getPort();
                String transport=viaHeader.getTransport();
                
                SipURI sipURI=addressFactory.createSipURI(null,host);
                sipURI.setPort(port);
                sipURI.setTransportParam(transport);
                Address peerAddress=addressFactory.createAddress(sipURI);
               
                RouteHeader routeHeader=headerFactory.createRouteHeader(peerAddress);
                routes.addElement(routeHeader);
                
                return routes;
            }
            else return null;
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public void processOK(Response responseCloned,ClientTransaction clientTransaction) {
         DebugIM.println("Processing OK for MESSAGE in progress...");
            
         InstantMessagingGUI instantMessagingGUI=imUA.getInstantMessagingGUI();
         ListenerInstantMessaging listenerInstantMessaging=
         instantMessagingGUI.getListenerInstantMessaging();
         ChatSessionManager chatSessionManager=listenerInstantMessaging.getChatSessionManager();
         ChatSession chatSession=null;
         String fromURL=IMUtilities.getKey(responseCloned,"To");
         if (chatSessionManager.hasAlreadyChatSession(fromURL)) {
             chatSession=chatSessionManager.getChatSession(fromURL);
             chatSession.displayLocalText();
             // WE remove the text typed:
             chatSession.removeSentText();
             
             // We need to store the Remote tag for the first OK
             String remoteTag=chatSession.getRemoteTag();
             if (remoteTag==null) {
                 try{
                     ToHeader toHeader=(ToHeader)responseCloned.getHeader(ToHeader.NAME);
                     String toTag=toHeader.getTag();
                     chatSession.setRemoteTag(toTag);
                     
                     // We need to store the Route header for building a next message
                     Vector routes=getRoutes(responseCloned);
                     if (routes!=null && !routes.isEmpty())
                        chatSession.setRoutes(routes);
                     
                     DebugIM.println("DEBUG, IMProcessing, processOK(), The remote tag is:"+toTag);
                 }
                 catch(Exception e) {
                     DebugIM.println("IMMessageProcessing, processOK(), exception raised:");
                     e.printStackTrace();
                 }
             }
         }
         else {
             // This is a bug!!!
             DebugIM.println("Error: IMMessageProcessing, processOK(), the chatSession is null");
         }
         DebugIM.println("Processing OK for MESSAGE completed...");
            
    }
    


     public void processMessage(Request request,ServerTransaction serverTransaction) {
        try{
            MessageFactory messageFactory=imUA.getMessageFactory();
            SipProvider sipProvider=imUA.getSipProvider();
            InstantMessagingGUI instantMessagingGUI=imUA.getInstantMessagingGUI();
            ListenerInstantMessaging listenerInstantMessaging=
            instantMessagingGUI.getListenerInstantMessaging();
            ChatSessionManager chatSessionManager=listenerInstantMessaging.getChatSessionManager();
            ChatSession chatSession=null;
            String fromURL=IMUtilities.getKey(request,"From");
            if (chatSessionManager.hasAlreadyChatSession(fromURL))
                chatSession=chatSessionManager.getChatSession(fromURL);
            else chatSession=chatSessionManager.createChatSession(fromURL);
            
            
            DebugIM.println("Process MESSAGE in progress...");
            
            // Send an OK
            Response response=messageFactory.createResponse
				(Response.OK,request);
            sipProvider.sendResponse(serverTransaction,response);
            DebugIM.println("OK replied to the MESSAGE");
                    
            String remoteTag=chatSession.getRemoteTag();
            CallIdHeader callIdHeader=(CallIdHeader)request.getHeader(CallIdHeader.NAME);
            if (!chatSession.equalsCallId(callIdHeader.getCallId()) ) {
                // we need to reinitialize the session because the GUY sent a BYE
                // and an other message just after!!!
                chatSession.init();
                chatSession.setInfo("");
            }
            
            // We need to store the Route header for building a next message
            Vector routes=getRoutes(request);
            if (routes!=null && !routes.isEmpty())
                chatSession.setRoutes(routes);
            
            remoteTag=chatSession.getRemoteTag();
            if (remoteTag==null) {
                // he initiates the session
                try{
                    DebugIM.println("DEBUG, IMMessageProcessing, processMessage(), we initialize"+
                    " the session with HIS callId and remoteTag");
                    ToHeader toHeader=(ToHeader)request.getHeader(ToHeader.NAME);
                    String toTag=toHeader.getTag();
                    chatSession.setRemoteTag(toTag);
                    
                    FromHeader fromHeader=(FromHeader)request.getHeader(FromHeader.NAME);
                    String fromTag=fromHeader.getTag();
                    chatSession.setLocalTag(fromTag);
                    
                    
                    chatSession.setCallId(callIdHeader.getCallId());
                    chatSession.setRemoteInitiateSession();
                    
                }
                catch(Exception e) {
                    DebugIM.println("IMMessageProcessing, processMessage(), exception raised:");
                    e.printStackTrace();
                }
            }
            
            Object content=request.getContent();
            String text=null;
            if (content instanceof String)
                text=(String)content;
            else
                if (content instanceof byte[] ) {
                    text=new String(  (byte[])content  );
                }
                else {
                }
            if (text!=null){
                chatSession.displayRemoteText(text);
            }
             
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    public void sendMessage(String localSipURL,String remoteSipURL,String text,
        ChatSession chatSession) {
        try {
            DebugIM.println();
            DebugIM.println("Sending a MESSAGE in progress to "+remoteSipURL);
  
            SipProvider sipProvider=imUA.getSipProvider();
            MessageFactory messageFactory=imUA.getMessageFactory();
            HeaderFactory headerFactory=imUA.getHeaderFactory();
            AddressFactory addressFactory=imUA.getAddressFactory();
            
            
            String proxyAddress=imUA.getProxyAddress();
            SipURI requestURI=null;
            if (proxyAddress!=null) {
                requestURI=addressFactory.createSipURI(null,proxyAddress);
                requestURI.setPort(imUA.getProxyPort());
                requestURI.setTransportParam(imUA.getIMProtocol());
            }
            else {
                requestURI=(SipURI)addressFactory.createURI(remoteSipURL);
                requestURI.setTransportParam(imUA.getIMProtocol());
            }
            
            CallIdHeader callIdHeader=null;
            
            // CSeq:
            cseq++;
            CSeqHeader cseqHeader=headerFactory.createCSeqHeader(cseq,"MESSAGE");
            
            // To header:
            Address toAddress=addressFactory.createAddress(remoteSipURL);
            ToHeader toHeader=null;
            
            // From Header:
            Address fromAddress=addressFactory.createAddress(localSipURL);
            FromHeader fromHeader=null;
            
            //  Via header
            String branchId=Utils.generateBranchId();
            ViaHeader viaHeader=headerFactory.createViaHeader(
                imUA.getIMAddress(),imUA.getIMPort(),imUA.getIMProtocol(),branchId);
            Vector viaList=new Vector();
            viaList.addElement(viaHeader);
            
            // MaxForwards header:
            MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(10);
            
            
            if (chatSession.hasExited()) {
                  chatSession.init();
            }
            
            String remoteTag=chatSession.getRemoteTag();
            if (remoteTag==null) {
                DebugIM.println("DEBUG, IMMessageProcessing, sendMessage(), we initialize"+
                    " the session with OUR callId and localTag");
                // We have to initiate the dialog: means to create the From tag
                String localTag=Utils.generateTag();
                chatSession.setLocalTag(localTag);
                fromHeader=headerFactory.createFromHeader(fromAddress,localTag);
                toHeader=headerFactory.createToHeader(toAddress,null);
                
                // Call-ID:
                callIdCounter++;
                callIdHeader=(CallIdHeader)headerFactory.createCallIdHeader(
                "nist-sip-im-message-callId"+callIdCounter);
               
                chatSession.setCallId("nist-sip-im-message-callId"+callIdCounter);
                chatSession.setLocalInitiateSession();
            }
            else {
                String callId=chatSession.getCallId();
                callIdHeader=(CallIdHeader)headerFactory.createCallIdHeader(callId);
            
                String localTag=chatSession.getLocalTag();
                fromHeader=headerFactory.createFromHeader(fromAddress,localTag);
                toHeader=headerFactory.createToHeader(toAddress,remoteTag);
            }
            
            // Content-Type:
            ContentTypeHeader contentTypeHeader=headerFactory.createContentTypeHeader(
            "text","plain");
            contentTypeHeader.setParameter("charset","UTF-8");
            
            Request request=messageFactory.createRequest(requestURI,"MESSAGE",
            callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader
            ,contentTypeHeader,text);
            ClientTransaction clientTransaction=sipProvider.getNewClientTransaction(request);
            
            // Contact header:
            SipURI sipURI=addressFactory.createSipURI(null,imUA.getIMAddress());
            sipURI.setPort(imUA.getIMPort());
            sipURI.setTransportParam(imUA.getIMProtocol());
            Address contactAddress=addressFactory.createAddress(sipURI);
            ContactHeader contactHeader=headerFactory.createContactHeader(contactAddress);
            request.setHeader(contactHeader);
           
            // we have to add the Route header if it exists:
            //Vector routes=chatSession.getRoutes();
            //if (routes!=null && !routes.isEmpty())
            //        request.setHeaders(routes);
            
            sipProvider.sendRequest(clientTransaction,request);
            
            DebugIM.println("MESSAGE sent:\n"+request);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

}
