/*
 * IMNotifyProcessing.java
 *
 * Created on September 26, 2002, 12:14 AM
 */

package examples.instantmessaging.presence;

import gov.nist.javax.sip.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;
import examples.instantmessaging.*;
import examples.instantmessaging.presence.pidfparser.*;
import examples.instantmessaging.presence.cpimparser.*;
/**
 *
 * @author  olivier
 * @version 1.0
 */
public class IMNotifyProcessing {

     private IMUserAgent imUA;
     private int cseq;
     public XMLpidfParser xmlPidfParser;
     public XMLcpimParser xmlCpimParser;
    
    /** Creates new IMNotifyProcessing */
    public IMNotifyProcessing( IMUserAgent imUA) {
        this.imUA=imUA;
        xmlPidfParser=new XMLpidfParser();
        xmlCpimParser=new XMLcpimParser();
        cseq=0;
    }

   
    public void processOk(Response response,ClientTransaction clientTransaction) {
        DebugIM.println("Processing OK received for a NOTIFY ");
        // We have to particular processing to do with the OK..
        DebugIM.println("OK processed!!!");
    }
    
    
    public void processNotify(Request request,ServerTransaction serverTransaction) {
        try {
            InstantMessagingGUI imGUI=imUA.getInstantMessagingGUI();
            ListenerInstantMessaging listenerIM=
            imGUI.getListenerInstantMessaging();
            MessageFactory messageFactory=imUA.getMessageFactory();
            SipProvider sipProvider=imUA.getSipProvider();
            
            DebugIM.println("Process NOTIFY in progress...");
            String fromURL=IMUtilities.getKey(request,"From");
            ExpiresHeader expiresHeader=(ExpiresHeader)request.getHeader(ExpiresHeader.NAME);
            if (expiresHeader!=null && expiresHeader.getExpires()==0) { 
                DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                " this is the NOTIFY related to the \"unSUBSCRIBE\"...");
                // We have to update the buddy list!!!
                    BuddyList buddyList=imGUI.getBuddyList();
                    buddyList.changeBuddyStatus(fromURL,"offline");
                    
                    // We can update the information field for the ChatFrame:
                    ChatSessionManager chatSessionManager=listenerIM.getChatSessionManager();
                    ChatSession chatSession=chatSessionManager.getChatSession(fromURL);
                    if (chatSession==null) {
                             DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                    " the chat session does not exist, no need to update the chatFrame!!!");
                    }
                    else { 
                            DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                    " the chat session does exist,  need to update the chatFrame!!!");
                            
                            chatSession.setInfo("The contact is offline");
                    }
                    
                    // Send an OK
                    Response response=messageFactory.createResponse
				(Response.OK,request);
                    sipProvider.sendResponse(serverTransaction,response);
                    DebugIM.println("OK replied to the NOTIFY");
                                      
                    // WE have to update the presentity list: the status has changed!!!
                    PresenceManager presenceManager=imUA.getPresenceManager();
                    presenceManager.updatePresentity(fromURL,"offline");
                    
                    // Very important: we need to subscribe again!!!
                    IMSubscribeProcessing imSubscribeProcessing=imUA.getIMSubscribeProcessing();
                    imSubscribeProcessing.sendSubscribe(listenerIM.getLocalSipURL(),fromURL,false);
            }
            else {
                
                Object content=request.getContent();
                String text=null;
                if (content instanceof String)
                    text=(String)content;
                else 
                    if (content instanceof byte[] ) {
                        text=new String(  (byte[])content  );
                    }
                    else {
                          DebugIM.println("DEBUG, IMNotifyProcessing, process(), "+
                          " Error, the body of the request is unknown!!");
                          DebugIM.println("ERROR, IMNotifyProcessing, process(): "+
                        " pb with the xml body, 488 Not Acceptable Here replied");
                        Response response=messageFactory.createResponse
				(Response.NOT_ACCEPTABLE,request);
                        sipProvider.sendResponse(serverTransaction,response);
                            return;
                    }
                    
                if (text!=null && !text.trim().equals("") ) {
                    // we have to parse the XML body!!!!
                    try{
                        ContentTypeHeader contentTypeHeader=(ContentTypeHeader)
                        request.getHeader(ContentTypeHeader.NAME);
                        String xmlType=contentTypeHeader.getContentSubType(); 
                        
                        DebugIM.println("DEBUG, IMNotifyProcessing, process(), the XML body format"+
                        " is: "+xmlType);
                        String status=null;
                        if (xmlType.equals("xpidf+xml") ) {
                            xmlPidfParser.parsePidfString(text);
                            examples.instantmessaging.presence.pidfparser.PresenceTag
                            presenceTag=xmlPidfParser.getPresenceTag();
                            if (presenceTag==null)
                                 DebugIM.println("ERROR: The presence Tag is null!!!");
                            else
                                DebugIM.println("the parsed body:"+presenceTag.toString());
                            
                            Vector atomTagList=presenceTag.getAtomTagList();
                            AtomTag atomTag=(AtomTag)atomTagList.firstElement();
                            AddressTag addressTag=atomTag.getAddressTag();
                            MSNSubStatusTag msnSubStatusTag=addressTag.getMSNSubStatusTag();
                            status=msnSubStatusTag.getMSNSubStatus();
                        }
                        else 
                            if (xmlType.equals("cpim-pidf+xml") ) {
                               
                                xmlCpimParser.parseCPIMString(text.trim());
                                examples.instantmessaging.presence.cpimparser.PresenceTag 
                                presenceTag=xmlCpimParser.getPresenceTag();
                                if (presenceTag==null)
                                    DebugIM.println("ERROR: The presence Tag is null!!!");
                                else
                                    DebugIM.println("the parsed body:"+presenceTag.toString());
                                
                                Vector tupleTagList=presenceTag.getTupleTagList();
                                TupleTag tupleTag=(TupleTag)tupleTagList.firstElement();
                                NoteTag noteTag=tupleTag.getNoteTag();
                                if (noteTag!=null)
                                    status=noteTag.getNote();
                                else status="offline";
                                DebugIM.println("status:"+status);
                            }
                        
                        
                        // Send an OK
                        Response response=messageFactory.createResponse
                        (Response.OK,request);
                        sipProvider.sendResponse(serverTransaction,response);
                        DebugIM.println("OK replied to the NOTIFY");
                        
                        
                        // We have to update the buddy list!!!
                        BuddyList buddyList=imGUI.getBuddyList();
                        buddyList.changeBuddyStatus(fromURL,status);
                        
                        // We can update the information field for the ChatFrame:
                        ChatSessionManager chatSessionManager=listenerIM.getChatSessionManager();
                        ChatSession chatSession=chatSessionManager.getChatSession(fromURL);
                        if (chatSession==null) {
                            DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                            " the chat session does not exist, no need to update the chatFrame!!!");
                        }
                        else {
                            DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                            " the chat session does exist,  need to update the chatFrame!!!");
                            chatSession.setInfo("The contact is "+status);
                            
                        }
                        
                        // WE have to update the presentity list: the status has changed!!!
                        PresenceManager presenceManager=imUA.getPresenceManager();
                        presenceManager.updatePresentity(fromURL,status);
                        
                    }
                    catch(Exception e) {
                        //e.printStackTrace();
                        DebugIM.println("ERROR, IMNotifyProcessing, process(): "+
                        " pb with the xml body, 488 Not Acceptable Here replied");
                        Response response=messageFactory.createResponse
				(Response.NOT_ACCEPTABLE,request);
                        sipProvider.sendResponse(serverTransaction,response);
                   
                        e.printStackTrace();
                    }
                }
                else {
                    DebugIM.println("DEBUG, IMNotifyProcessing, processNotify(), "+
                    " PB to get the NOTIFY xml body, 488 Not Acceptable Here replied");
                   
                    // Send an OK
                    Response response=messageFactory.createResponse
                        (Response.OK,request);
                    sipProvider.sendResponse(serverTransaction,response);
                    DebugIM.println("OK replied to the NOTIFY");
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    public void sendNotifyToAllSubscribers(String status,String subStatus) {
        try{
             // We have to get all our subscribers and send them a NOTIFY!
             PresenceManager presenceManager=imUA.getPresenceManager();
             Vector subscribersList=presenceManager.getAllSubscribers();
             DebugIM.println("DEBUG, IMNotifyProcessing, sendNotifyToAllSuscribers(),"+
             " we have to notify our SUBSCRIBERS: let's send a NOTIFY for each one "+
             "of them (subscribersList: "+subscribersList.size()+")!!!");
             for (int i=0;i<subscribersList.size();i++) {
                Subscriber subscriber=(Subscriber)subscribersList.elementAt(i); 
                
                Response okSent=subscriber.getOkSent();
                String subscriberName=subscriber.getSubscriberName();
                
                String contactAddress= imUA.getIMAddress()+":"+
                                       imUA.getIMPort();
                String xmlBody=null;
                if (!status.equals("closed") )
                    xmlBody=xmlPidfParser.createXMLBody(status,subStatus,subscriberName,contactAddress);
                
                sendNotify(okSent,xmlBody);
             }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        } 
    }
    
    
    public void sendNotify(Response okSent,String body) {
        try{
            // We send the NOTIFY!!!
            
            // we create the Request-URI: the one of the proxy
            HeaderFactory headerFactory=imUA.getHeaderFactory();
            AddressFactory addressFactory=imUA.getAddressFactory();
            MessageFactory messageFactory=imUA.getMessageFactory();
            SipProvider sipProvider=imUA.getSipProvider();
            
            String imProtocol=imUA.getIMProtocol();
            String proxyAddress=imUA.getProxyAddress();
            int proxyPort=imUA.getProxyPort();
            
            SipURI requestURI=null;
            if (proxyAddress!=null) {
                 requestURI=addressFactory.createSipURI(null,proxyAddress);
                 requestURI.setPort(proxyPort);
                 requestURI.setTransportParam(imProtocol);
            }
            else {
                DebugIM.println("DEBUG, IMNotifyProcessing, sendNotify(), request-uri is null");
                return;
            }
            
            CallIdHeader callIdHeader=(CallIdHeader)okSent.getHeader(CallIdHeader.NAME);
            
            // CSeq:
            cseq++;
            CSeqHeader cseqHeader=headerFactory.createCSeqHeader(cseq,"NOTIFY");
            
            // FromHeader
            ToHeader to=(ToHeader)okSent.getHeader(ToHeader.NAME);
            Address toAddress=to.getAddress();
            FromHeader fromHeader=headerFactory.createFromHeader(toAddress,to.getTag());
         
            
            // ToHeader
            FromHeader from=(FromHeader)okSent.getHeader(FromHeader.NAME);
            Address fromAddress=from.getAddress();
            ToHeader toHeader=headerFactory.createToHeader(fromAddress,from.getTag());
            
             //  Via header
            String branchId=Utils.generateBranchId();
            ViaHeader viaHeader=headerFactory.createViaHeader(
                imUA.getIMAddress(),imUA.getIMPort(),imProtocol,branchId);
            Vector viaList=new Vector();
            viaList.addElement(viaHeader);
            
            // MaxForwards header:
            MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(10);
            
            Request request=null;
            ClientTransaction clientTransaction=null;
            if (body==null) {
                
                request=messageFactory.createRequest(requestURI,"NOTIFY",
                callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader);
                clientTransaction=sipProvider.getNewClientTransaction(request);
            
                // We have to add an Expires-Header of 0!!!
                ExpiresHeader expiresHeader=headerFactory.createExpiresHeader(0);
                request.setHeader(expiresHeader);
                
            }
            else {
                body=body+"\r\n";
                // Content-Type:
                ContentTypeHeader contentTypeHeader=headerFactory.createContentTypeHeader(
                "application","xpidf+xml");
                
                request=messageFactory.createRequest(requestURI,"NOTIFY",
                callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader
                ,contentTypeHeader,body);
                clientTransaction=sipProvider.getNewClientTransaction(request);
                        
            }
         
            // WE have to add a new Header: "Subscription-State"
            DebugIM.println("DEBUG, IMNotifyProcessing, sendNotify(), We add the Subscription-State"+
            " header to the request");
            Header header=headerFactory.createHeader("Subscription-State","active");
            request.setHeader(header);
            
            // WE have to add a new Header: "Event"
            header=headerFactory.createHeader("Event","presence");
            request.setHeader(header);
            
            sipProvider.sendRequest(clientTransaction,request);
            DebugIM.println("DEBUG, IMNotifyProcessing, sendNotify(),"+
            " NOTIFY sent:\n" );
            DebugIM.println(request.toString());
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
}
