/*
 * IMSubscribeProcessing.java
 *
 * Created on September 26, 2002, 12:13 AM
 */

package examples.instantmessaging.presence;

import gov.nist.javax.sip.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import examples.instantmessaging.*;
/**
 *
 * @author  olivier
 * @version 1.0
 */
public class IMSubscribeProcessing {

    private IMUserAgent imUA;
    private int callIdCounter;
    private int cseq;

    /** Creates new IMSubscribeProcessing */
    public IMSubscribeProcessing(IMUserAgent imUA) {
        this.imUA=imUA;
        callIdCounter=0;
        cseq=0;
    }
    
    
    public void processOK(Response responseCloned,ClientTransaction clientTransaction) {
        try{
            DebugIM.println("Processing OK for SUBSCRIBE in progress...");
            
            ExpiresHeader expiresHeader=(ExpiresHeader)responseCloned.getHeader(ExpiresHeader.NAME);
            if (expiresHeader!=null && expiresHeader.getExpires()==0) { 
                DebugIM.println("DEBUG, IMSubscribeProcessing, processOK(), we got"+
                " the OK for the unsubscribe...");
            }
            else { 
               
                // We have to create or update the presentity!
                PresenceManager presenceManager=imUA.getPresenceManager();
                String presentityURL=IMUtilities.getKey(responseCloned,"To");
                
                presenceManager.addPresentity(presentityURL,responseCloned);
                
                // WE have to create a new Buddy in the GUI!!!
                InstantMessagingGUI imGUI=imUA.getInstantMessagingGUI();
                BuddyList buddyList=imGUI.getBuddyList();
                if ( !buddyList.hasBuddy(presentityURL) ) {
                    buddyList.addBuddy(presentityURL,"offline");
                }
                else {
                    DebugIM.println("The buddy is already in the Buddy list...");
                }
            }
            DebugIM.println("Processing OK for SUBSCRIBE completed...");
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
   
    
    public void processSubscribe(Request request,ServerTransaction serverTransaction) {    
        DebugIM.println("Processing SUBSCRIBE in progress ");
        try {
            MessageFactory messageFactory=imUA.getMessageFactory();
            HeaderFactory headerFactory=imUA.getHeaderFactory();
            AddressFactory addressFactory=imUA.getAddressFactory();
            /************************ OK for the SUBSCRIBE **********************/
            // We have to ask the user to authorize the guy to be in his buddy list
            String presentityURL=IMUtilities.getKey(request,"From");
            
            SipProvider sipProvider=imUA.getSipProvider();
            
            InstantMessagingGUI imGUI=imUA.getInstantMessagingGUI();
            boolean authorization=imGUI.getAuthorizationForBuddy(presentityURL);
            if ( authorization ) {
                DebugIM.println("DEBUG: SubscribeProcessing, processSubscribe(), "+
                " Response 200 OK sent.");
             
                // Send an OK
                Response response=messageFactory.createResponse
				(Response.OK,request);
                sipProvider.sendResponse(serverTransaction,response);
                
                // We have to create or update the subscriber!
                PresenceManager presenceManager=imUA.getPresenceManager();
                String subscriberURL=IMUtilities.getKey(request,"From");
              
                presenceManager.addSubscriber(subscriberURL,response);
                
                // Let's see if this buddy is in our buddy list
                // if not let's ask to add him!
                BuddyList buddyList=imGUI.getBuddyList();
                ListenerInstantMessaging listenerIM=imGUI.getListenerInstantMessaging();
                if ( !buddyList.hasBuddy(subscriberURL) ) {
                    // Let's ask:
                    listenerIM.addContact(subscriberURL);
                }

                DebugIM.print(response.toString()); 
                
                /************************ send NOTIFY **************************/
                // We send a NOTIFY for any of our status but offline
                String localStatus=listenerIM.getLocalStatus();
                if (!localStatus.equals("offline") ) {
                    Subscriber subscriber=presenceManager.getSubscriber(subscriberURL);
                    
                    Response okSent=subscriber.getOkSent();
                    subscriberURL=subscriber.getSubscriberName();
                    
                    String contactAddress= imUA.getIMAddress()+":"+
                    imUA.getIMPort();
                    
                    String subStatus=listenerIM.getLocalStatus();
                    String status=null;
                    if (subStatus.equals("offline")) 
                        status="closed";
                    else status="open";
                    
                    IMNotifyProcessing imNotifyProcessing=imUA.getIMNotifyProcessing();
                    String xmlBody=imNotifyProcessing.xmlPidfParser.createXMLBody(status,subStatus,subscriberURL,contactAddress);
                    
                    
                    imNotifyProcessing.sendNotify(okSent,xmlBody);
                }
            }
            else {
                DebugIM.println("DEBUG, IMSubsribeProcessing, processSubscribe(), "+
                " Subscription declined!");
                DebugIM.println("DEBUG, IMSubsribeProcessing, processSubscribe(), "+
                " 603 Decline replied");
                Response response=messageFactory.createResponse
				(Response.DECLINE ,request);
                sipProvider.sendResponse(serverTransaction,response);
                
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
          
    
    public void sendSubscribeToAllPresentities(Vector buddies,boolean EXPIRED) {
        try{
             DebugIM.println("DebugIM, IMSubscribeProcessing, sendSubscribeToAllPresentities(),"+
             " we have to subscribe to our buddies: let's send a SUBSCRIBE for each ones.");
             for (int i=0;i<buddies.size();i++) {
                BuddyTag buddyTag=(BuddyTag)buddies.elementAt(i);
                
                String buddyURI=buddyTag.getURI();
                
                InstantMessagingGUI imGUI=imUA.getInstantMessagingGUI();
                ListenerInstantMessaging listenerIM=imGUI.getListenerInstantMessaging();
                String localURL=listenerIM.getLocalSipURL();
                sendSubscribe(localURL,buddyURI,EXPIRED);
             }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        } 
    }
    
    public void sendSubscribe(String localURL,String buddyURI,boolean EXPIRED) {
        try {
            DebugIM.println();
            DebugIM.println("Sending SUBSCRIBE in progress to the buddy: "+buddyURI);
            int proxyPort=imUA.getProxyPort();
            String proxyAddress=imUA.getProxyAddress();
            String imProtocol=imUA.getIMProtocol();
            SipStack sipStack=imUA.getSipStack();
            SipProvider sipProvider=imUA.getSipProvider();
            MessageFactory messageFactory=imUA.getMessageFactory();
            HeaderFactory headerFactory=imUA.getHeaderFactory();
            AddressFactory addressFactory=imUA.getAddressFactory();
            
            
            // Request-URI:
           // URI requestURI=addressFactory.createURI(buddyURI);
            SipURI requestURI=addressFactory.createSipURI(null,proxyAddress);
            requestURI.setPort(proxyPort);
            requestURI.setTransportParam(imProtocol);
            
            CallIdHeader callIdHeader=null;
            
            // CSeq:
            cseq++;
            CSeqHeader cseqHeader=headerFactory.createCSeqHeader(cseq,"SUBSCRIBE");
            
            // To header:
            Address toAddress=addressFactory.createAddress(buddyURI);
            ToHeader toHeader=null;
            
            // From Header:
            Address fromAddress=addressFactory.createAddress(localURL);
            FromHeader fromHeader=null;
            
            //  Via header
            String branchId=Utils.generateBranchId();
            ViaHeader viaHeader=headerFactory.createViaHeader(
                imUA.getIMAddress(),imUA.getIMPort(),imProtocol,branchId);
            Vector viaList=new Vector();
            viaList.addElement(viaHeader);
            
            
            PresenceManager presenceManager=imUA.getPresenceManager();
            if (EXPIRED) {
                // we will unsubscribe: expires time=0
                // WE have to retrieve the tags, callId
                Presentity presentity=presenceManager.getPresentity(buddyURI);
                if (presentity==null) {
                    new Exception("ERROR, IMSubscribeProcessing, sendSubscribe, "+
                    "The presentity "+buddyURI+" is not found, we cannot retrieve the tags and CallId..."+
                    " The unSUBSCRIBE is not sent!!!");
                }
                else {
                    Response response=presentity.getOkReceived();
                    
                    ToHeader to=(ToHeader)response.getHeader(ToHeader.NAME);
                    String remoteTag=to.getTag();
                    fromHeader=headerFactory.createFromHeader(fromAddress,remoteTag);
                    
                    FromHeader from=(FromHeader)response.getHeader(FromHeader.NAME);
                    String localTag=from.getTag();
                    toHeader=headerFactory.createToHeader(toAddress,localTag);
                   
                    CallIdHeader call=(CallIdHeader)response.getHeader(CallIdHeader.NAME);
                    String callId=call.getCallId();
                    // Call-ID:
                    callIdHeader=(CallIdHeader)headerFactory.createCallIdHeader(callId);
                
                    // We have to remove the subscriber and the Presentity related
                    // with this Buddy...
                    presenceManager.removePresentity(buddyURI);
                    Subscriber subscriber=presenceManager.getSubscriber(buddyURI);
                    if (subscriber==null ) {
                        // It means that the guy does not have us in his buddy list
                        // nothing to do!!!
                    }
                    else {
                         presenceManager.removeSubscriber(buddyURI);
                    }
                }
            }
            else{
                String localTag=Utils.generateTag();
                fromHeader=headerFactory.createFromHeader(fromAddress,localTag);
                    
                toHeader=headerFactory.createToHeader(toAddress,null);
                
                callIdCounter++;
                // Call-ID:
                callIdHeader=(CallIdHeader)
                headerFactory.createCallIdHeader("nist-sip-im-subscribe-callId"+callIdCounter);
               
            }
            
            // MaxForwards header:
            MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(10);
            
            Request request=messageFactory.createRequest(requestURI,"SUBSCRIBE",
            callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader);
            ClientTransaction clientTransaction=sipProvider.getNewClientTransaction(request);
            
            // Contact header:
            SipURI sipURI=addressFactory.createSipURI(null,imUA.getIMAddress());
            sipURI.setPort(imUA.getIMPort());
            sipURI.setTransportParam(imUA.getIMProtocol());
            Address contactAddress=addressFactory.createAddress(sipURI);
            ContactHeader contactHeader=headerFactory.createContactHeader(contactAddress);
            request.setHeader(contactHeader);
            
            
            ExpiresHeader expiresHeader=null;
            if (EXPIRED) {
                expiresHeader=headerFactory.createExpiresHeader(0);
            }
            else {
                expiresHeader=headerFactory.createExpiresHeader(presenceManager.getExpiresTime());
            }
            request.setHeader(expiresHeader);
            
            // WE have to add a new Header: "Event"
            Header header=headerFactory.createHeader("Event","presence");
            request.setHeader(header);
            
            sipProvider.sendRequest(clientTransaction,request);
            DebugIM.println("IMSubscribeProcessing, sendSubscribe(), SUBSCRIBE sent:\n"+
            request);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
}
