
package examples.proxy;

import examples.proxy.*;
import examples.proxy.registrar.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;
/** Process bye requests.
*
*@version  JAIN-SIP-1.1
*
*@author Olivier Deruelle <deruelle@nist.gov>  <br/>
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/
public class ByeProcessing  extends DefaultProcessing {
	
   
    /** Policy: Always forward the message following the Dialogs
     */
    public void processRequest
	(Proxy proxy,Request request,ServerTransaction serverTransaction) {
        try {
            SipProvider sipProvider=proxy.getSipProvider();
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            MessageFactory messageFactory=proxy.getMessageFactory();
            TransactionsMapping
            transactionsMapping=proxy.getTransactionsMapping();
            
            if (ProxyDebug.debug)
                ProxyDebug.println("Processing BYE in progress.");
            
            Response response=messageFactory.createResponse
            (Response.OK,request);
            sipProvider.sendResponse(serverTransaction,response);
            if (ProxyDebug.debug)
                ProxyDebug.println("OK replied to the BYE:\n"+response.toString());
            
            
            Dialog dialog = serverTransaction.getDialog();
            
            Transaction transaction = dialog.getFirstTransaction();
            
            Dialog outgoingDialog = null;
            
            if (ProxyDebug.debug)
                ProxyDebug.println("First transaction retrieved from dialog=" + transaction);
            
            if (transaction instanceof ClientTransaction) {
                ClientTransaction ct = (ClientTransaction)transaction;
                ServerTransaction st =
                transactionsMapping.getServerTransaction(ct);
                
                if (ProxyDebug.debug)
                ProxyDebug.println("Server transaction retrieved from the first"+
                " transaction=" + st);
                
                if (st == null) {
                    if (ProxyDebug.debug)
                        ProxyDebug.println
			("Could not find the server transaction, we drop the request!");
		   return;
		}
                outgoingDialog = st.getDialog();
            } else {
                ServerTransaction st = (ServerTransaction) transaction;
                // Other side of the call leg.
                ClientTransaction ct =
                transactionsMapping.getClientTransaction(st);
                if (ProxyDebug.debug)
                ProxyDebug.println("Client transaction retrieved from the first"+
                " transaction=" + ct);
                if (ct == null) {
                    if (ProxyDebug.debug)
                        ProxyDebug.println
			("Could not find the client transaction, we drop the request!");
		   return;
		}
		outgoingDialog = ct.getDialog();
	   }

	   if (ProxyDebug.debug)
               ProxyDebug.println
	    	("ByeProcessing: dialog retrieved:"+ outgoingDialog);
		
           if (outgoingDialog == null)   {
                ProxyDebug.println
	      ("ByeProcessing: could not find outgoing dialog ");
	      return;
	   }
	
	   Request inviteRequest = 
		outgoingDialog.getFirstTransaction().getRequest();
	   Request byeRequest = messageFactory.createByeRequest
				(inviteRequest, outgoingDialog.isServer());
	   if (byeRequest.getHeader(MaxForwardsHeader.NAME) == null)  {
	       MaxForwardsHeader mf  = 
		  proxy.headerFactory.createMaxForwardsHeader(70);
		byeRequest.addHeader(mf);
	    }
	    
	   
	   ListIterator it = request.getHeaders(RouteHeader.NAME);
		
	   // Copy the route from the incoming BYE
	   if (it != null)  {
		RouteHeader topRoute = null;
		while (it.hasNext()) {
		   RouteHeader route = (RouteHeader) it.next();
		   String host = 
			((SipURI)route.getAddress().getURI()).getHost();
		   int port = 
			((SipURI)route.getAddress().getURI()).getPort();
		   if ( checkHostPort(proxy,host,port)) continue;
		   if (topRoute == null)  {
			topRoute = (RouteHeader)
			((RouteHeader) route).clone();
		   } 
		   byeRequest.addHeader((RouteHeader) route.clone());
		}
		if (topRoute != null) {
		    SipURI sipURI = (SipURI) topRoute.getAddress().getURI();
		    byeRequest.setRequestURI(sipURI);
		}
	        System.out.println("byeRequest = " + byeRequest);
	   }
	   

           ClientTransaction ct=
		sipProvider.getNewClientTransaction(byeRequest);
	if (ct == null) {
            ProxyDebug.println
            ("ByeProcessing: could not route request -- dropping ");
        }
           transactionsMapping.addMapping(serverTransaction,ct);
           outgoingDialog.sendRequest(ct,byeRequest);
           
           if (ProxyDebug.debug) {
               ProxyDebug.println("BYE forwarded with:\n");
               if (ct.getDialog()!=null)
                   ProxyDebug.println(
                   "- dialog Id:"+ct.getDialog().getDialogId());
               else   ProxyDebug.println("- dialog Id: null");
               ProxyDebug.println("\n- clientTransaction branch Id:"+
               ct.getBranchId() );
           }
           
        }
        catch (Exception ex) {
            ex.printStackTrace();
            if (ProxyDebug.debug)
                ProxyDebug.println
		   ("ByeProcessing, process(), exception raised:");
        }
    }
    
    
}
