package examples.proxy;

import java.util.*;
import examples.proxy.registrar.*;
import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
/** Processing for Cancel Requests.
 *
 *@version  JAIN-SIP-1.1
 *
 *@author Olivier Deruelle<deruelle@nist.gov>  <br/>
 *@author M. Ranganathan <mranga@nist.gov>  <br/>
 *
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 */
public class CancelProcessing extends DefaultProcessing {
    
    public void processRequest(Proxy proxy,
    Request request,ServerTransaction serverTransaction){
        try {
            
            SipProvider sipProvider=proxy.getSipProvider();
            Registrar registrar=proxy.getRegistrar();
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            MessageFactory messageFactory=proxy.getMessageFactory();
            TransactionsMapping transactionsMapping=
            proxy.getTransactionsMapping();
            if (ProxyDebug.debug)
                ProxyDebug.println("Processing CANCEL in progress");
            
            // 487 Request Terminated
            Response response=messageFactory.createResponse
            (Response.REQUEST_TERMINATED,request);
            sipProvider.sendResponse(serverTransaction,response);
            ProxyDebug.println("487 created and replied to the CANCEL:\n" +
                    response);
            // wait to forward the 200 OK from the other peer.
            //response=messageFactory.createResponse(Response.OK,request);
            //sipProvider.sendResponse(serverTransaction,response);
            
            Dialog dialog = serverTransaction.getDialog();
            Vector clientTransactions = null;
            
            if (dialog != null && dialog.isServer()) {
                ServerTransaction st = (ServerTransaction)
                dialog.getFirstTransaction();
                clientTransactions =
                transactionsMapping.getClientTransactions(st);
            }
            if ( clientTransactions != null ) {
                
                for (Enumeration e = clientTransactions.elements(); 
			e.hasMoreElements();) {
                    ClientTransaction ct = (ClientTransaction)e.nextElement();
                    
                    // check if the client transaction can be canceled.
                    if (ct.getState().equals(TransactionState.COMPLETED) ||
                    ct.getState().equals(TransactionState.TERMINATED)){
                        continue;
                    }
                    Request cancelRequest
                    = messageFactory.createCancelRequest(ct.getRequest());   
                    
                 
                    
                    ClientTransaction clientTransaction=
                    sipProvider.getNewClientTransaction
                    (cancelRequest);
                    sipProvider.sendRequest
                    (clientTransaction,cancelRequest);
                    ProxyDebug.println("CancelProcessing, CANCEL forwarded:  " +
                    cancelRequest);
		    // Note - DO NOT add this to the transaction table.
                    
                }
            } else {
                // Statelessly forward the cancel request.
	       if (ProxyDebug.debug) 
		   ProxyDebug.println("CancelProcessing, forwarding CANCEL statelessly");
               
               request = proxyUtilities.addProxyViaHeader
			(request,"udp",serverTransaction);
               
               super.forwardStatelessly(proxy,request,serverTransaction);    
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
}
