/*
 * Configuration.java
 *
 * Created on February 3, 2003, 3:21 PM
 */

package examples.proxy;
import java.util.*;
/**
*
*@version  JAIN-SIP-1.1
*
*@author Olivier Deruelle <deruelle@nist.gov>  <br/>
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public class Configuration {
    
    // Stack:
    public String stackName;
    public String stackIPAddress;
    public String outboundProxy;
    public String routerPath;
    public String extensionMethods;
    public String retransmissionFilter;
    public String stopTime;
    public String maxConnections;
    public String maxServerTransactions;
    public String threadPoolSize;
    public String stackDomain;
    
    // Registrar
    public String  registryPort;
    public boolean exportRegistry;
    
    // Logging
    public boolean accessLogViaRMI;
    public String logRMIPort;
    public String logLifetime;
    public boolean enableDebug;
    public String serverLogFile;
    public String badMessageLogFile;
    public String debugLogFile;
    public String outputProxy;
    
    // Presence server
    public boolean enablePresenceServer;
    
    // Authentication
    public boolean enableAuthentication;
    public String method;
    public String classFile;
    public String passwordsFile;
    
    // Registrations
    public boolean enableRegistrations;
    public long expiresTime=3600;
    public String registrationsFile;
    
    // Listening points:
    public Hashtable listeningPoints;
    
     // methods processing:
    protected Hashtable methodsProcessing;
    protected Hashtable methodsInstances;
    
    private int counter;
    
    /** Creates a new instance of Configuration */
    public Configuration() {
        counter=0;
        listeningPoints=new Hashtable();
        methodsProcessing=new Hashtable();
    }
    
 
    
    public void addListeningPoint(String port,String transport) {
        if (check(port) && check(transport) ) {
            counter++;
            listeningPoints.put
		("listeningPoint"+counter,new Association(port,transport) );
        }
    }
    
    protected void addMethodProcessing(String methodName,String className) {
        if (check(methodName) && check(className) ) {
            methodsProcessing.put(methodName.toUpperCase(),className);
           
        }
    }
    
    public Vector getListeningPoints() {
        if (listeningPoints!=null) {
            Collection c = listeningPoints.values();
           return new Vector(c);
        }
        return null;
    }
    
    public MethodProcessing getMethodProcessing(String methodName) {
        if (methodName!=null) {
           return (MethodProcessing)
		methodsInstances.get(methodName.toUpperCase());
        }
        return null;
    }
    
    
    protected boolean check(String s) {
        return  (s!=null && !s.trim().equals(""));
    }
    
    protected void initMethodsProcessing() {
        Enumeration keys=methodsProcessing.keys();
        methodsInstances=new Hashtable();
	methodsInstances.put("INVITE",new InviteProcessing());
	methodsInstances.put("REGISTER",new RegisterProcessing());
	methodsInstances.put("OPTIONS",new OptionsProcessing());
	methodsInstances.put("BYE", new ByeProcessing());
	methodsInstances.put("ACK",new AckProcessing());
	methodsInstances.put("CANCEL",new CancelProcessing());
        methodsInstances.put("DEFAULT_PROCESSING",new DefaultProcessing());
        methodsInstances.put("NOTIFY",new NotifyProcessing());
	methodsInstances.put("SUBSCRIBE",new SubscribeProcessing());
        methodsInstances.put("MESSAGE",new MessageProcessing());
	// Add extension methods here.
        while( keys.hasMoreElements() ) {
            String methodName=(String)keys.nextElement();
            String className=(String)methodsProcessing.get(methodName);
            
            try{
                Class methodClass =Class.forName(className);
                MethodProcessing methodProcessing
                = (MethodProcessing)
                methodClass.newInstance();
                methodsInstances.put(methodName.toUpperCase(),methodProcessing);
                System.out.println("Successfull class uploading for the "+
                methodName.toUpperCase()+
                " method processing!");
            }
            catch(Exception e) {
                System.out.println
		("WARNING, there is no class file for processing the "+
                methodName.toUpperCase()+" messages; exception raised:");
                e.printStackTrace();
            }
        }       
    }
    
    
    public boolean isValidConfiguration() {
        // Just a warning check for the basics methods
       
       if (enableAuthentication) {
            if ( check(method) &&
                 check(classFile) && 
                 check(passwordsFile) ) {
                 // OK
            }
            else {
                System.out.println
		("ERROR, the configuration is not valid: Problem with"+
                " the authentication tag and authentication parameters");
                return false;
            }
       }
        
       if  ( check(stackName) &&
             check(stackIPAddress) &&
             !listeningPoints.isEmpty()
             ) {
             return true;
       }
       else {
           System.out.println
		("ERROR, the configuration is not valid: Problem with"+
                " the stack tag and stack parameters");
            return false;
       }
       
       
    }
    
}
