package examples.proxy;
import java.util.*;
import examples.proxy.registrar.*;
import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;

/** Default method processing.
 *
 *@version  JAIN-SIP-1.1
 *
 *@author Olivier Deruelle <deruelle@nist.gov>  
 *@author M. Ranganathan <mranga@nist.gov>  <br/>
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 */
public class DefaultProcessing implements MethodProcessing{

    protected boolean checkHostPort(Proxy proxy, String host, int port) {
        if (proxy.getSipStack().getIPAddress().equals(host))  {
            Iterator lps=proxy.sipStack.getListeningPoints();
            if (lps==null) return false;
            while(lps.hasNext()) {
                ListeningPoint lp=(ListeningPoint)lps.next();
                if (lp.getPort()==port) return true;
            }
            return false;
	} else return false;
    }
    
    /** If the request has from AND to tag, we retrieve the parent
     * dialog and we get the first transaction, then we check if it
     * is a server or client transaction and we get the mapping from
     * the table, get its dialog and get the tags!
     *
     * If the request has ONLY from tag, we create a new From tag
     * and forward the request!
     **/
    public Request updateTags(Proxy proxy,Request request,ServerTransaction serverTransaction) {
        try{
            ToHeader toHeader=(ToHeader)request.getHeader(ToHeader.NAME);
            FromHeader fromHeader=(FromHeader)request.getHeader(FromHeader.NAME);
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            
            if (fromHeader.getTag()!=null && toHeader.getTag()==null) {
                ProxyDebug.println("DefaultProcessing, updateTags(), the request "+
                "is the first transaction for the future dialog, we have to create"+
                " a new From tag!" );
                fromHeader.setTag(proxyUtilities.generateTag());
                return request;
            }
            
            TransactionsMapping transactionsMapping =
            proxy.getTransactionsMapping();
            Dialog parentDialog=serverTransaction.getDialog();
            if (parentDialog==null ) {
                ProxyDebug.println("DefaultProcessing, updateTags(), the parent dialog"+
                " is null, the method is not dialog creator!! We can not update the tags" );
                return request;
            }
            else 
                if (parentDialog.getState()==null) {
                     ProxyDebug.println("DefaultProcessing, updateTags(), the parent dialog"+
                     " exists, but the state is null! We can not update the tags" );
                }
                else {
                     ProxyDebug.println("DefaultProcessing, updateTags(), the parent dialog"+
                     " exists, and the state is not null!" );
                }
            
            Transaction transaction=parentDialog.getFirstTransaction();
            if (transaction==null)  {
                ProxyDebug.println("DefaultProcessing, updateTags(), the first "+
                "transaction is null, we can not update the tags" );
                return request;
            }
            
            String fromTag=null;
            String toTag=null;
            if (transaction instanceof ServerTransaction) {
                ServerTransaction st=(ServerTransaction)transaction;
                ClientTransaction ct=transactionsMapping.getClientTransaction(st);
                Dialog dialog=ct.getDialog();
                fromTag=dialog.getLocalTag();
                toTag=dialog.getRemoteTag();
                ProxyDebug.println("DefaultProcessing, updateTags(), we use the tags"+
                " (the transaction is server) from the dialog: fromTag="+fromTag+" , toTag="+toTag);
            }
            else {
                ClientTransaction ct=(ClientTransaction)transaction;
                ServerTransaction st=transactionsMapping.getServerTransaction(ct);
                Dialog dialog=st.getDialog();
                toTag=dialog.getLocalTag();
                fromTag=dialog.getRemoteTag();
                ProxyDebug.println("DefaultProcessing, updateTags(), we use the tags"+
                 " (the transaction is client) from the dialog: fromTag="+fromTag+" , toTag="+toTag);
            }
            
            fromHeader.setTag(fromTag);
            toHeader.setTag(toTag);
            
            return request;
            
        }
        catch (Exception ex) {
            ex.printStackTrace();
            return request;
        }
    }
    
    
    /** The default processing of messages.
     * Look up all the locations to which the original invite transaction
     * is forwarded and send the request there.
     */
    public void processRequest(Proxy proxy,Request request,ServerTransaction serverTransaction) {
        Registrar registrar = proxy.getRegistrar();
        ProxyUtilities proxyUtilities = proxy.getProxyUtilities();
        SipProvider sipProvider = proxy.getSipProvider();
        TransactionsMapping transactionsMapping =
        proxy.getTransactionsMapping();
        MessageFactory messageFactory = proxy.getMessageFactory();
        try {
            if ( registrar.hasRegistration(request) ) {
                Vector contacts=registrar.getContacts(request);
                
                if (contacts==null || contacts.isEmpty()) {
                    if (ProxyDebug.debug)
                        ProxyDebug.println
                        ("DefaultProcessing: no contact headers to forward to...");
                    if (ProxyDebug.debug)
                        ProxyDebug.println(
                        "DefaultProcessing: 480 TEMPORARILY_NOT_AVAILABLE replied");
                    Response response=messageFactory.createResponse
                    (Response.TEMPORARILY_UNAVAILABLE,request);
                    sipProvider.sendResponse(serverTransaction,response);
                    return;
                }
                
               
                for (int i = 0; i < contacts.size(); i++) {
                    Request requestCloned=(Request)request.clone();
                    ContactHeader contact = (ContactHeader)
                    contacts.elementAt(i);
                    requestCloned=proxyUtilities.updateRequest(requestCloned,
                    (SipURI)contact.getAddress().getURI(),serverTransaction);
                    
                    /** If the request has from AND to tag, we retrieve the parent
                     * dialog and we get the first transaction, then we check if it
                     * is a server or client transaction and we get the mapping from
                     * the table, get its dialog and get the tags!
                     *
                     * If the request has ONLY from tag, we create a new From tag
                     * and forward the request!
                     **/
                    requestCloned=updateTags(proxy,requestCloned,serverTransaction);
                    
                    if (requestCloned!=null) {
                        ClientTransaction ct =
                        sipProvider.getNewClientTransaction
                        (requestCloned);
                        sipProvider.sendRequest(ct,requestCloned);
                        
                        ProxyDebug.println
                        ("DEBUG, DefaultProcessing, processRequest()"+
                        ", request forwarded:\n "
                        +requestCloned);
                        
                        transactionsMapping.addMapping(serverTransaction,ct);
                    }
                    else {
                        ProxyDebug.println("ERROR, DefaultProcessing, processRequest()"+
                        ", PB to forward the request...:\n ");
                    }
                }
            }
            else {
                
                // Add the Via Header:
                request=proxyUtilities.addProxyViaHeader(request,"UDP",serverTransaction);
                
                
                // We have to check if a user is registered with the domain of the request
                // URI, otherwise we use our hops:
                if (registrar.hasDomainRegistered(request)) {
                    request=(Request)proxyUtilities.addProxyRecordRoute(request,"UDP");
                    forwardRequestToDomain(proxy,request,serverTransaction);
                    return;
                }
                else
                    forwardStatelessly(proxy,request,serverTransaction);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    public void forwardRequestToDomain
    (Proxy proxy,Request request,ServerTransaction serverTransaction) {
        try {
            
            Registrar registrar=proxy.getRegistrar();
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            SipProvider sipProvider=proxy.getSipProvider();
            MessageFactory messageFactory=proxy.getMessageFactory();
            TransactionsMapping transactionsMapping=
            proxy.getTransactionsMapping();
               
            Vector contacts=registrar.getDomainContacts(request);
            
            if (contacts==null || contacts.isEmpty()) return;
            
            
            ContactHeader contact=(ContactHeader)contacts.firstElement();
            if (contact!=null ) {
                String tag=ProxyUtilities.generateTag();
                
                Request requestCloned=(Request)request.clone();
                
                if (requestCloned!=null) {
                  
                    URI requestURI=requestCloned.getRequestURI();
                    if (requestURI.isSipURI()) {
                        SipURI requestSipURI=(SipURI)requestURI;
                        String user=requestSipURI.getUser();
                        String host=requestSipURI.getHost();
                        
                        Address address=contact.getAddress();
                        URI contactURI=address.getURI();
                       
                        if ( !contactURI.isSipURI()) 
                            return;
                        
                        SipURI contactSipURI=(SipURI)contactURI;
                        requestSipURI.setHost(contactSipURI.getHost());
                        requestSipURI.setPort(contactSipURI.getPort());
                        
                         ProxyDebug.println("Forward to Domain, "+
                    "request ready to be sent:\n "+requestCloned.toString());
                        
                        ClientTransaction clientTransaction=
                        sipProvider.getNewClientTransaction
                        (requestCloned);
                    
                        
                        sipProvider.sendRequest
                        (clientTransaction,requestCloned);
                        
                        transactionsMapping.addMapping
                        (serverTransaction,clientTransaction);
                    }
                }
                else {
                    ProxyDebug.println("ERROR, DefaultProcessing(), "+
                    "PB to forward the REQUEST to the Domain...\n ");
                }
            }
        }
        catch (Exception ex) {
            ProxyDebug.println("ERROR, DefaultProcessing(), "+
            "PB to forward the REQUEST to the Domain...\n ");
            ex.printStackTrace();
        }
      }

    
    protected void forwardStatelessly(Proxy proxy, Request request,
    ServerTransaction serverTransaction ) throws Exception {
        SipProvider sipProvider = proxy.sipProvider;
        AddressFactory addressFactory = proxy.addressFactory;
        HeaderFactory headerFactory=proxy.headerFactory;
        MessageFactory messageFactory=proxy.messageFactory;
        Registrar registrar=proxy.getRegistrar();
        ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
        
    
        if (request.getHeader(RouteHeader.NAME) != null) {
            // forward the request
            // to the specified route header.
            sipProvider.sendStatelessRequest(request);
            ProxyDebug.println("REQUEST forwarded statelessly (using"+
            "the Route header):\n "
            +request);
            return;
        }
        
        
        Router router = proxy.sipStack.getRouter();
        
        if (request.getHeader(RouteHeader.NAME) != null) {
            // Already has a route header so just forward the request
            // to the specified route header.
            sipProvider.sendStatelessRequest(request);
            if (ProxyDebug.debug)
                ProxyDebug.println("request forwarded statelessly using the ROUTE header:\n "
                +request);
        } else {
           
            Hop hop = router.getOutboundProxy();
            if (hop==null ) {
                if (ProxyDebug.debug)
                    ProxyDebug.println("No hop to forward the request to. We reply"+
                    " 404 NOT FOUND");
                 Response response = messageFactory.createResponse
                (Response.NOT_FOUND,request);
                sipProvider.sendResponse(serverTransaction,response);
                return;
            }
            
          
            Request requestCloned = (Request) request.clone();
            SipURI uri = proxy.addressFactory.createSipURI
            (null,hop.getHost());
            uri.setPort(hop.getPort());
            Address address =
            proxy.addressFactory.createAddress(uri);
            RouteHeader route =
            proxy.headerFactory.createRouteHeader(address);
            requestCloned.addHeader(route);
            sipProvider.sendStatelessRequest(requestCloned);
            if (ProxyDebug.debug)
                ProxyDebug.println("Default Route used to forward the request statelessly:\n "
                +requestCloned);
            
        }

    }

        /** JAIN Listener method.
     */
    public void processResponse
    (Proxy proxy, Response response,ClientTransaction clientTransaction) {
        try{
           CSeqHeader cseqHeader=(CSeqHeader)
            response.getHeader(CSeqHeader.NAME);
            if  (response.getStatusCode() == 100) {
                if (ProxyDebug.debug)  {
                    ProxyDebug.println("100 Trying response - dropping");
                }
                return;
            } else if (response.getStatusCode()==487){
                if (ProxyDebug.debug) {
                    ProxyDebug.println
                        ("This message is the 487 REQUEST TERMINATED" +
                        " related to the CANCEL, we don't"+
                        " forward it because it has already been sent " +
                        " to the other side");
                }
                return; 
            } else if ( cseqHeader.getMethod().equals("BYE") ) {
                if (ProxyDebug.debug) {
                    ProxyDebug.println
                        ("This message is the OK related to the BYE, ignore"+
                         " it has already been sent to the other side");
                }
                return ;
            }
            /*else if ( cseqHeader.getMethod().equals("INFO") ) {
                if (ProxyDebug.debug) {
                    ProxyDebug.println
                             ("This message is the OK related to the "+
			    " INFO, we ignore " +
                            " it because it has already been sent to "+
			    " the other side");
                }
                return;
            }
            */
          
            
            Response responseCloned=(Response)response.clone();
            
            // We have to forward the response if it is not for the proxy,
            // therefore, if no Via headers remain in the response
            // after removing the topmost Via header.
            ListIterator viaList=responseCloned.getHeaders(ViaHeader.NAME);
            if (viaList==null || !viaList.hasNext()) { 
                ProxyDebug.println("ERROR, the response does not contain any "+
                "VIA headers");
                return;
            }
            else {
                viaList.next();
                viaList.remove();
            }
            
           
            if (ProxyDebug.debug) {
                proxy.transactionsMapping.printTransactionsMapping();
            }
            
            if (clientTransaction == null)  {
                if (ProxyDebug.debug)  {
                    ProxyDebug.println("The client transaction does not exist, we"+
                    " forward the response statelessly!");
                }
                 proxy.sipProvider.sendStatelessResponse(responseCloned);
                 return;
            }
            
        
            if ( viaList.hasNext() ) {
                try {
                    
                    ServerTransaction serverTransaction=proxy.transactionsMapping.
                    getServerTransaction(clientTransaction);
                    if (serverTransaction == null) {
                        ProxyDebug.println("ERROR, could not find mapping: "+
                        "serverTransaction is null, request dropped!");
                        return;
                    }
                    
                    // TAGS business!!!
                    /*  If the peer dialog does not exist for the transaction
                     *  then the response is the first final one: the from
                     *  tag will be assigned by the stack and we generate a new To tag.
                     *
                     *  if the peer dialog exist, then we do nothing, the stack will
                     *  manage the tags.
                     */
                    Dialog dialog = serverTransaction.getDialog();
                    if ( dialog!=null && dialog.getState()==null ) {
                         if (ProxyDebug.debug)
                            ProxyDebug.println
                            ("ProcessResponse, the server dialog is null, this response is a first"+
                            " final response");
                          ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            
                          ToHeader to=(ToHeader)responseCloned.getHeader(ToHeader.NAME);
                          to.setTag(proxyUtilities.generateTag());
                    }
                    
                
                    if  (responseCloned.getStatusCode() == 486 ) {
                        
                        // We have to check if it is a busy and we have multiple
                        // pending transactions:
                        Vector clientsTransactionList=proxy.transactionsMapping.
                        getClientTransactions(serverTransaction);
                        // We have to cancel all the others transactions:
                        if (clientsTransactionList.size()>=2) {
                            // We have to reply ACK:
                            
                            Request inviteRequest=clientTransaction.getRequest();
                            
                            URI requestURI=inviteRequest.getRequestURI();
                            CallIdHeader callId=(CallIdHeader)inviteRequest.getHeader(CallIdHeader.NAME);
                            CSeqHeader cSeq=(CSeqHeader)response.getHeader(CSeqHeader.NAME);
                            FromHeader from=(FromHeader)inviteRequest.getHeader(FromHeader.NAME);
                            ToHeader to=(ToHeader)inviteRequest.getHeader(ToHeader.NAME);
                            ListIterator via=response.getHeaders(ViaHeader.NAME);
                            Vector viaVector=new Vector();
                            while (via!=null && via.hasNext()) {
                                viaVector.addElement(via.next());
                            }
                            
                            MaxForwardsHeader maxForwards=(MaxForwardsHeader)inviteRequest.
                            getHeader(MaxForwardsHeader.NAME);
                            
                            Request request=proxy.messageFactory.createRequest(requestURI,Request.ACK,callId,
                            cSeq,from,to, viaVector,maxForwards);
                            
                            proxy.sipProvider.sendRequest(clientTransaction,request);
                            if (ProxyDebug.debug)
                                ProxyDebug.println
                                ("ACK request replied to BUSY:\n"+request);
                            
                            return;
                        }
                    }
                          
                    if  (responseCloned.getStatusCode() != 200 ) {
                     
                        proxy.sipProvider.sendResponse
                        (serverTransaction,responseCloned);
                        if (ProxyDebug.debug)
                            ProxyDebug.println
                            ("Response forwarded:\n"+responseCloned);
                        
                        if ( dialog!=null && dialog.getState()!=null ) {
                         if (ProxyDebug.debug)
                            ProxyDebug.println
                            ("The server dialog state is now:"+dialog.getState().toString());
                        }
                        
                        return;
                    }
                    
                    
                    
                    // Let's check if the server transaction is confirmed:
                    // If no, we have to forward the OK, and send CANCEL to 
		    // the others.
                    // If yes, we don't forward it and reply BYE to the OK.
                    TransactionState ts=serverTransaction.getState();
                    if ( ts==null ||
                    ts.getValue()==TransactionState.TRYING.getValue() ||
                    ts.getValue()==TransactionState.PROCEEDING.getValue() ||
                    ts.getValue()==TransactionState.CALLING.getValue() ) {
                        // Send the response on its merry way.
                       
                        proxy.sipProvider.sendResponse
                        (serverTransaction,responseCloned);
                         if (ProxyDebug.debug)
                            ProxyDebug.println
                            ("Response forwarded:\n"+responseCloned);
                        
                        Vector clientsTransactionList=proxy.transactionsMapping.
                        getClientTransactions(serverTransaction);
                        // We have to cancel all the others transactions:
                        for (int i=0;i<clientsTransactionList.size();i++) {
                            ClientTransaction ctr=(ClientTransaction)
                            clientsTransactionList.elementAt(i);
                            if ( ctr != clientTransaction ) {
                                
                                Request cancelRequest
                                = proxy.messageFactory.createCancelRequest
				    (ctr.getRequest());
                                ClientTransaction ct=
                                proxy.sipProvider.getNewClientTransaction
                                (cancelRequest);
                                proxy.sipProvider.sendRequest
                                (ct,cancelRequest);
                                ProxyDebug.println
				("CANCEL created and sent:  " +
                                cancelRequest);
                                
                                 // Send the BYE:
                                Dialog clientDialog=ctr.getDialog();
                                
                                ProxyDebug.println("ctr:"+ctr);
				ProxyDebug.println("ctr request = " + ctr.getRequest());
				ProxyDebug.println("dialog = " + clientDialog);

                                Request byeRequest
                                = proxy.messageFactory.createByeRequest(
                                ctr.getRequest(),
                                false);
				byeRequest.removeHeader(ViaHeader.NAME);
				proxy.getProxyUtilities().updateRequest
				(byeRequest, clientDialog);
                                ProxyDebug.println("BYE created:  " +
                                byeRequest);
                                ct=
                                proxy.sipProvider.getNewClientTransaction
                                (byeRequest);
                                try{
                                    clientDialog.sendRequest(ct,byeRequest);
                                }
                                catch(Exception e) {}
                                ProxyDebug.println("BYE created and sent:  " +
                                byeRequest);
                                
                                proxy.transactionsMapping.removeMapping(ctr);
                            }
                        }
                    } else {
                        
                         Vector clientsTransactionList=proxy.transactionsMapping.
                        getClientTransactions(serverTransaction);
                        if (clientsTransactionList==null || clientsTransactionList.size()==1){
                             ProxyDebug.println("This OK is a retransmission, just forward:");
                              proxy.sipProvider.sendResponse
                        (serverTransaction,responseCloned);
                         if (ProxyDebug.debug)
                            ProxyDebug.println
                            ("Response forwarded:\n"+responseCloned);
                            return;
                        }
                         
                        // Send the BYE:
                        Dialog clientDialog=clientTransaction.getDialog();
                        Request byeRequest
                        = proxy.messageFactory.createByeRequest(
                        clientTransaction.getRequest(),
                        false);
                        try{
                            clientDialog.sendRequest(clientTransaction,byeRequest);
                        }
                        catch(Exception e) {}
                        
                        ProxyDebug.println("BYE created and sent:  " +
                        byeRequest);
                        
                        proxy.transactionsMapping.removeMapping(clientTransaction);
                    }
                    
                } 
                catch (Exception ex) {
                    ProxyDebug.println("Unable to process the response:");
                    ex.printStackTrace();
                    return;
                }
            } else {
                // The response is for the proxy..
                if (ProxyDebug.debug)
                     ProxyDebug.println
                     ("We just discard this response targeted to the proxy.");
            }
            
            if (ProxyDebug.debug)
                ProxyDebug.println
                ("****************************************************");
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
}


