package examples.proxy;

import java.util.*;
import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import examples.proxy.presenceserver.*;
import examples.proxy.registrar.*;
/** Processing for notify requests.
*
*@version  JAIN-SIP-1.1
*
*@author Olivier Deruelle <deruelle@antd.nist.gov>  <br/>
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/
public class NotifyProcessing extends DefaultProcessing {


    
    public void processResponse(Proxy proxy,
	Response response,ClientTransaction clientTransaction) {
        ProxyDebug.println
		("Processing OK received for a NOTIFY (Presence Server)");
        // We don't forward the OK related to the NOTIFY...
    }
    
    /** Policy: we just
     * use our presence server, otherwise we forward the NOTIFY
     */
    public void  processRequest
    (Proxy proxy,Request request,ServerTransaction serverTransaction){
        try{
            
             if (!proxy.isPresenceServer()) {
                ProxyDebug.println("Processing NOTIFY in progress (Presence Server "+
                "disabled: default processing)");
                super.processRequest(proxy,request,serverTransaction);
                return;
            }
            
            PresenceServer presenceServer=proxy.getPresenceServer();
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            MessageFactory messageFactory=proxy.getMessageFactory();
            Registrar registrar=proxy.getRegistrar();
            HeaderFactory headerFactory=proxy.getHeaderFactory();
            SipProvider sipProvider=proxy.getSipProvider();
            TransactionsMapping transactionsMapping =
            proxy.getTransactionsMapping();
            ToHeader toHeader=(ToHeader)request.getHeader(ToHeader.NAME);
            String toTag=toHeader.getTag();
            if (toTag==null) {
                ProxyDebug.println
                ("ERROR, NotifyProcessing, process(), the NOTIFY request" +
                " has not been forwarded because the tags were not set " +
                "correctly!!!");
                ProxyDebug.println
                ("NotifyProcessing, process(), 481 " +
                "Call-leg/Transaction does not exist replied");
                
                Response response=messageFactory.createResponse
                (Response.CALL_OR_TRANSACTION_DOES_NOT_EXIST,request);
                sipProvider.sendResponse(serverTransaction,response);
                return;
            }
            
            ProxyDebug.println("Processing NOTIFY request. " +
            " The presence server is going to process it, "  +
            " and send it to all the subscribers");
            
            /*********************** We forward the NOTIFY  ******************/
            String presentity=presenceServer.getKey(request,"To");
            Vector contacts=registrar.getContacts(request);
                
            if ( registrar.hasRegistration(presentity) && contacts!=null &&
                 !contacts.isEmpty()) {
              
                ProxyDebug.println("DEBUG: NotifyProcessing, processRequest(), "+
                " The contact is registered, we forward the NOTIFY");
                
                for (int i = 0; i < contacts.size(); i++) {
                    
                    ContactHeader contact = (ContactHeader)
                    contacts.elementAt(i);
                    
                    Request requestCloned=(Request)request.clone();
                    
                    requestCloned=proxyUtilities.updateRequest(requestCloned,
                    (SipURI)contact.getAddress().getURI(),serverTransaction);
                    
                    if (requestCloned!=null) {
                        requestCloned=presenceServer.updateTags(proxy,requestCloned,serverTransaction);
                    }
                    
                    if (requestCloned!=null) {
                        ClientTransaction ct =
                        sipProvider.getNewClientTransaction
                        (requestCloned);
                        sipProvider.sendRequest(ct,requestCloned);
                        
                        ProxyDebug.println
                        ("DEBUG, NotifyProcessing, processRequest()"+
                        ", request forwarded:\n "
                        +requestCloned);
                        
                        transactionsMapping.addMapping(serverTransaction,ct);
                    }
                    else {
                        ProxyDebug.println("ERROR, NotifyProcessing, processRequest()"+
                        ", PB to forward the request...:\n ");
                    }
                }
            }
            
        }
        catch(Exception e) {
            ProxyDebug.println("ERROR, NotifyProcessing, process(), exception raised:");
            e.printStackTrace();
        }
    }
    /*
  
    public void sendNotify(Request subscribeRequest,Response okResponse,String body,
    String subscriptionState) {
        try{
            // WE have to get the Contact header from the registration for the
            // guy that sent the Subscribe and put it in the Request-URI !!
            HeaderFactory headerFactory=proxy.getHeaderFactory();
            AddressFactory addressFactory=proxy.getAddressFactory();
            MessageFactory messageFactory=proxy.getMessageFactory();
            SipProvider sipProvider=proxy.getSipProvider();
            SipStack sipStack=proxy.getSipStack();
            PresenceServer presenceServer=proxy.getPresenceServer();
            String fromURL=presenceServer.getKey(subscribeRequest,"From");
            Registrar registrar=proxy.getRegistrar();
            
            if ( registrar.hasRegistration(fromURL) ) {
                Vector contacts=registrar.getContactHeaders(fromURL);
                
                if (contacts==null || contacts.isEmpty()) {
                    ProxyDebug.println("DEBUG, NotifyProcessing, sendNotify():"+
                    "no contact headers to forward to...");
                }
                else {
                    // Request-URI for the NOTIFY
                    ContactHeader contactHeader=(ContactHeader)contacts.firstElement();
                    Address contactAddress=contactHeader.getAddress();
                    URI requestURI=contactAddress.getURI();
                    
                    // Call-Id header
                    CallIdHeader callIdHeader=(CallIdHeader)subscribeRequest.getHeader(CallIdHeader.NAME);
                    
                    // CSeq header
                    CSeqHeader cseq=(CSeqHeader)subscribeRequest.getHeader(CSeqHeader.NAME);
                    int cs=cseq.getSequenceNumber();
                    CSeqHeader cseqHeader=headerFactory.createCSeqHeader(cs,"NOTIFY");
                    
                    // FromHeader
                    ToHeader to=(ToHeader)okResponse.getHeader(ToHeader.NAME);
                    Address toAddress=to.getAddress();
                    FromHeader fromHeader=headerFactory.createFromHeader(toAddress,to.getTag());
                    
                    // ToHeader
                    FromHeader from=(FromHeader)okResponse.getHeader(FromHeader.NAME);
                    Address fromAddress=from.getAddress();
                    ToHeader toHeader=headerFactory.createToHeader(fromAddress,from.getTag());
                    
                    // Via header
                    String stackIPAddress=sipStack.getIPAddress();
                    Iterator lps=sipStack.getListeningPoints();
                    ListeningPoint lp=(ListeningPoint)lps.next();
                    ViaHeader viaHeader=headerFactory.createViaHeader(
                    stackIPAddress,lp.getPort(),lp.getTransport(),null);
                    Vector viaList=new Vector();
                    viaList.addElement(viaHeader);
                    
                    
                    // MaxForwards header:
                    MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(10);
                    
                    Request request=null;
                    ClientTransaction clientTransaction=null;
                    if (body==null) {
                        
                        request=messageFactory.createRequest(requestURI,"NOTIFY",
                        callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader);
                        clientTransaction=sipProvider.getNewClientTransaction(request);
                        
                        // We have to add an Expires-Header of 0!!!
                        ExpiresHeader expiresHeader=headerFactory.createExpiresHeader(0);
                        request.setHeader(expiresHeader);
                        
                    }
                    else {
                        
                        // Content-Type:
                        ContentTypeHeader contentTypeHeader=headerFactory.createContentTypeHeader(
                        "application","xpidf+xml");
                        
                        request=messageFactory.createRequest(requestURI,"NOTIFY",
                        callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader
                        ,contentTypeHeader,body);
                        clientTransaction=sipProvider.getNewClientTransaction(request);
                        
                    }
                    
                    // WE have to add a new Header: "Subscription-State"
                    ProxyDebug.println("DEBUG, NotifyProcessing, sendNotify(),"+
                    " we add the Subscription-State"+
                    " header to the request");
                    Header header=headerFactory.createHeader("Subscription-State",subscriptionState);
                    request.setHeader(header);
                    
                    sipProvider.sendRequest(clientTransaction,request);
                  
                    ProxyDebug.println("NOTIFY sent:\n"+request.toString());
            
                }
            }
            else {
                ProxyDebug.println("DEBUG, NotifyProcessing, sendNotify():"+
                "no registration, we don't send the NOTIFY");
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    */
}

