package examples.proxy;

import java.util.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
/** Utilities for the proxy.
*
*@version  JAIN-SIP-1.1
*
*@author Olvier Deruelle <deruelle@nist.gov>  <br/>
*@author M. Ranganathan <mranga@nist.gov> <br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/
public class ProxyUtilities {
 
    protected Proxy proxy;
    
    protected ProxyUtilities(Proxy proxy) {
        this.proxy=proxy;
    }
    
     /**
     * to hex converter
     */
    private static final char[] toHex = { '0', '1', '2', '3', '4', '5', '6',
                                          '7', '8', '9', 'a', 'b', 'c', 'd',
                                          'e', 'f' };
 
    /**
     * convert an array of bytes to an hexadecimal string
     * @return a string
     * @param b bytes array to convert to a hexadecimal
     * string
     */
 
    public static String toHexString(byte b[]) {
        int pos = 0;
        char[] c = new char[b.length*2];
        for (int i=0; i< b.length; i++) {
            c[pos++] = toHex[(b[i] >> 4) & 0x0F];
            c[pos++] = toHex[b[i] & 0x0f];
        }
        return new String(c);
    }  
    
    
    
    public static String generateTag() {
            return new Integer((int)(Math.random() * 10000)).toString();
    }
    
    protected boolean matchProxyAddress(Request request) throws Exception{
        javax.sip.address.URI uri=request.getRequestURI();
        SipStack sipStack=proxy.getSipStack();
        if (uri==null) return false;
        else {
            SipURI sipURI=((SipURI)uri);
            
            String stackIPAddress=sipStack.getIPAddress();
            String host=sipURI.getHost();
            
            return stackIPAddress.equals(host);
           
        }
    }
    
    protected int length(Request request) {
        int cpt=0;
        try{
            ListIterator l=request.getHeaders(ViaHeader.NAME);
            while (l.hasNext() ) {
                    l.next();
                    cpt++;
            }
            return cpt;
        }
        catch(Exception e) { return cpt;}
    }
    
    protected boolean hasViaHeaderProxy(SipStack sipStack,Request request) {
        ListIterator viaList=request.getHeaders(ViaHeader.NAME);
        if (viaList==null ||  length(request)<=1) 
               return false;
        
        String stackIPAddress=sipStack.getIPAddress();
        while (viaList.hasNext() ) {
           ViaHeader viaHeader=(ViaHeader) viaList.next();
          
        
          if (  viaHeader.getHost().equals(stackIPAddress) ) {
              Iterator lps=sipStack.getListeningPoints();
                while (lps!=null && lps.hasNext() ) {
                    ListeningPoint lp = (ListeningPoint)lps.next();
                    int port = lp.getPort();
                    if (viaHeader.getPort()==port)
                        return true;
                }
          }
        }
        return false;
    }
    
    
    protected  Request updateRequest(Request request,Dialog dialog) {
	Address address = dialog.getRemoteTarget();
        try{
            // WE have to put the protocol of the contact into the
            // Via header and Record-Route:
            javax.sip.address.SipURI
            contactURI=(javax.sip.address.SipURI)address.getURI();
            
            Request req=updateRequest
		(request,contactURI,dialog.getFirstTransaction());
            if (req==null) return null;
           
            return req;
            
        }
        catch (Exception ex) {
            ProxyDebug.println("ProxyUtilities, updateRequest()," +
			" exception raised:"+ex.getMessage());
            ex.printStackTrace();
            return null;
        }
    }
    
    public Request updateRequest
	(Request request, SipURI contactURI,Transaction transaction) {
        try{
           
            request.setRequestURI(contactURI);
            URI requestURI=request.getRequestURI();
            if (requestURI instanceof SipURI)
                ((SipURI)requestURI).setMAddrParam(contactURI.getHost());
            // WE have to put the protocol of the contact into the
            // Via header and Record-Route:
            String transport=contactURI.getTransportParam();
            if (transport==null) {
                 transport="udp";
            }
            Request req=addProxyViaHeader(request,transport,transaction);
            if (req==null) return null;
            req=(Request)addProxyRecordRoute(req,transport);
            
            
            return request;
        }
        catch (Exception ex) {
            ProxyDebug.println("ProxyUtilities, updateRequestURI()," +
			" exception raised:"+ex.getMessage());
            ex.printStackTrace();
            return null;
        }
    }
    
  
    protected Request addProxyViaHeader
	(Request request, String transport,
    	Transaction transaction) {
        try{
            
              
            // We have to add one Via header corresponding to the proxy
            SipStack sipStack=proxy.getSipStack();
            HeaderFactory headerFactory=proxy.getHeaderFactory();
            AddressFactory addressFactory=proxy.getAddressFactory();
            String stackIPAddress=sipStack.getIPAddress();
            Iterator lps=sipStack.getListeningPoints();
	    ListeningPoint lp = null;
	    while (lps.hasNext()) {
                lp=(ListeningPoint)lps.next();
		if (lp.getTransport().equalsIgnoreCase(transport)) break;
	    }
            
            ViaHeader viaHeader=null;
		
	    if (transaction == null) {
		// caller wants a stateless request.
                 viaHeader=headerFactory.createViaHeader
			(stackIPAddress,lp.getPort(),
                        transport,null);
	    } else if (! request.getMethod().equals(Request.CANCEL)) {
		// Branch Id will be assigned by the stack.
                 viaHeader=headerFactory.createViaHeader
			(stackIPAddress,lp.getPort(),
                        transport,null);
            } else  {
               //  retrieve the INVITE branch Id:
	        Request req = transaction.getRequest();
		String branchId = ((ViaHeader) req.getHeader
						(ViaHeader.NAME))
					    .getParameter("branch");
                        
                viaHeader=headerFactory.createViaHeader
				(stackIPAddress,lp.getPort(),
                        	transport,branchId);
		// Cancel is hop by hop so remove all other via headers.
		request.removeHeader(ViaHeader.NAME);
            }
            
            if (viaHeader!=null) 
                request.addHeader(viaHeader);
            else   ProxyDebug.println("ProxyUtilities, updateRequest()," +
			" PB to add the proxy via header");

            return request;
        }
        catch (Exception ex) {
            ProxyDebug.println("ProxyUtilities, updateRequest()," +
			" exception raised:"+ex.getMessage());
            ex.printStackTrace();
            return null;
        }
    }
 
    protected boolean hasNextHop(){
        SipStack sipStack=proxy.getSipStack();
        Router router=sipStack.getRouter();
        
        if (router!=null) {
            
            Hop hop=router.getOutboundProxy();
            
            return (hop!=null);
        }
        else return false;
    }

    protected Request getRequestNextHop(Request request)
    throws Exception {
        
        AddressFactory addressFactory=proxy.getAddressFactory();
        SipStack sipStack=proxy.getSipStack();
        Router router=sipStack.getRouter();
        Hop hop=router.getOutboundProxy();
        
        SipURI sipURI=addressFactory.createSipURI(null,hop.getHost());
        sipURI.setMAddrParam(hop.getHost());
        sipURI.setPort(hop.getPort());
        sipURI.setTransportParam(hop.getTransport());
        Address address=addressFactory.createAddress(null,sipURI);
        request.setRequestURI(sipURI);
        
        // The Record-Route header to add:
        Request req=(Request)addProxyRecordRoute(request,hop.getTransport());
        if (req==null) return null;
        
        // We have to add one Via header corresponding to the proxy
        req=addProxyViaHeader(req,hop.getTransport(),null);
	
        return req;
    }


    protected Message addProxyRecordRoute(Message message,String transport) 
	throws Exception{
        
        ListIterator recordRouteHeaders=message.getHeaders
		(RecordRouteHeader.NAME);
        message.removeHeader(RecordRouteHeader.NAME);
        Vector v=new Vector();
        
        // WE add the proxy RecordRoute header
        HeaderFactory headerFactory=proxy.getHeaderFactory();
        AddressFactory addressFactory=proxy.getAddressFactory();
        SipStack sipStack=proxy.getSipStack();
        String stackIPAddress=sipStack.getIPAddress();
        Iterator lps=sipStack.getListeningPoints();
        ListeningPoint lp=(ListeningPoint)lps.next();
        
        SipURI sipURI=addressFactory.createSipURI(null,stackIPAddress);
        sipURI.setPort(lp.getPort());
        sipURI.setTransportParam(transport);
        Address address=addressFactory.createAddress(null,sipURI);
        RecordRouteHeader recordRouteHeader=
            headerFactory.createRecordRouteHeader(address);
        
        message.addHeader(recordRouteHeader);
        // add the other record route headers.
        if (recordRouteHeaders!=null) {
            while( recordRouteHeaders.hasNext()) {
                recordRouteHeader=(RecordRouteHeader)
                recordRouteHeaders.next();
                message.addHeader(recordRouteHeader);
             
            }
        }
       
        
        return message;
    }
 
}
