package examples.proxy;

import java.util.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;

import examples.proxy.registrar.*;

/** A class that manages mapping of client to server transactions.
*
*@version  JAIN-SIP-1.1
*
*@author Olivier Deruelle <deruelle@nist.gov>  <br/>
*M. Ranganathan <mranga@nist.gov>  <br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public class TransactionsMapping implements Runnable {

    private  Hashtable table;
    
    /** Creates new TransactionsTable */
    protected TransactionsMapping() {
        table= new Hashtable();
	new Thread(this).start();
    }

    public synchronized ServerTransaction 
	getServerTransaction(ClientTransaction clientTransaction) {
        // Retrieve the good value:
        Enumeration e=table.keys();
      
        while( e.hasMoreElements() ) {
           ServerTransaction serverTransaction=
			(ServerTransaction)e.nextElement();
           Vector vector=(Vector)table.get(serverTransaction);
           
           for (Enumeration en = vector.elements(); en.hasMoreElements();){
                ClientTransaction clientTransac=
			(ClientTransaction)en.nextElement();
                if (clientTransac.getBranchId()==
			clientTransaction.getBranchId()) {
                        return serverTransaction;
                }
           }
        }
        return null;
    }
    
    public synchronized ClientTransaction 
	getClientTransaction(ServerTransaction serverTransaction) {
         Vector vector=(Vector)table.get(serverTransaction);
        if (vector == null) return null;
	else {
		for (Enumeration e = vector.elements() ; 
			e.hasMoreElements(); ) {
		    ClientTransaction ct = 
			(ClientTransaction) e.nextElement();
		    Dialog d = ct.getDialog();
		    if ( d.getState() != null && 
			  d.getState().equals(DialogState.CONFIRMED))
			  return ct;
		    else 
			if ( d.getState() == null) { 
			   ProxyDebug.println("TransactionsMapping, getClientTransaction(),"+
                           " the dialog state is null.");  
                        }
                        else {
                            ProxyDebug.println("TransactionsMapping, getClientTransaction(),"+
                           " the dialog state is:"+d.getState().toString());  
                        }
		}
		return null;
	 }
	 
    }
    
    protected synchronized boolean hasMapping(ServerTransaction st) {
        if (! table.containsKey(st)) return false;
        else {
            // retrieve the mapping from the table and check if is empty.
            Vector vector = (Vector) table.get(st);
            return ! vector.isEmpty();
        }
    }
    
    public synchronized Vector getClientTransactions
		(ServerTransaction serverTransaction) {
         return (Vector)table.get(serverTransaction);
    }
     
    public synchronized void 
	addMapping
	(ServerTransaction serverTransaction,
		ClientTransaction clientTransaction) {
         Vector clients=getClientTransactions(serverTransaction);
         if (clients==null) clients=new Vector();
         if (serverTransaction!=null && clientTransaction!=null) {
            clients.addElement(clientTransaction); 
            table.put(serverTransaction,clients);
         }
    }

    protected synchronized void removeMapping
	(ServerTransaction serverTransaction) {
	System.out.println("removing " + serverTransaction);
	table.remove(serverTransaction);

    }

    protected synchronized void removeMapping
	(ClientTransaction clientTransaction) {
	System.out.println("removing " + clientTransaction);
	ServerTransaction serverTransaction = 
		getServerTransaction(clientTransaction);
	Vector clientTransactions = getClientTransactions(serverTransaction);
	boolean toRemove = false;
	ClientTransaction ct = null;
	for ( Enumeration e = clientTransactions.elements() ; 
		e.hasMoreElements(); ) {
		ct = (ClientTransaction)  e.nextElement();
		  if (ct.getBranchId()==
			clientTransaction.getBranchId())  {
			toRemove = true;
			 break;
		   }
	}
	if (toRemove) {
		clientTransactions.remove(ct);
		if ( clientTransactions.isEmpty()) {
		   table.remove(serverTransaction);
		}
	}
    }
    
    
    protected synchronized void printTransactionsMapping(){
        ProxyDebug.println("Transactions table:"); 
        Enumeration e=table.keys();
      
        while( e.hasMoreElements() ) {
           ServerTransaction serverTransaction=
			(ServerTransaction)e.nextElement();
           ProxyDebug.println("serverTransaction: "
			+serverTransaction);
           Vector vector=(Vector)table.get(serverTransaction);
           
           for (int i=0;i<vector.size();i++){
                ClientTransaction clientTransac=
			(ClientTransaction)vector.elementAt(i);
                ProxyDebug.println
		("   clientTransaction: "+clientTransac);
           }
        }
	if (ProxyDebug.debug) ProxyDebug.println();
    }
	
    /** Clean up the mapping table 
    */
    public void run() {
	while(true) {
		try {
			Thread.sleep(2000);
		} catch (InterruptedException ex) {  }
		synchronized (table) {
        	   Enumeration e=table.keys();
		   while(e.hasMoreElements()) {
           		ServerTransaction serverTransaction=
			       (ServerTransaction)e.nextElement();
                        
			if (serverTransaction.getDialog() == null) {
                        
                            if (ProxyDebug.debug)  {
				   ProxyDebug.println
					("removing " + serverTransaction + 
					" because its dialog is null" );
				   table.remove(serverTransaction);
				}
                        }
                         /*
                         
		        } else if (serverTransaction.getDialog().getState() 
			       != null && serverTransaction.getDialog().
			      getState().equals(DialogState.TERMINATED)) {
				 ProxyDebug.println
				     ("removing" + serverTransaction + 
					" dialogState = "  + 
				      serverTransaction.getDialog().getState());
			         table.remove(serverTransaction);
			} else {
			     if(ProxyDebug.debug) 
				ProxyDebug.println("dialog state = " +
				serverTransaction.getDialog().getState() + 
				" method = " +
				serverTransaction.getRequest().getMethod());
			}
                             */
                            
		}
	}
      }
   }
		
    
}
