/*
 * PresenceServer.java
 *
 * Created on September 20, 2002, 3:46 PM
 */

package examples.proxy.presenceserver;

import java.util.*;
import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import examples.proxy.*;
import examples.proxy.registrar.*;

/**
 *
 * @author  deruelle
 * @version 1.0
 */
public class PresenceServer {

    protected int expiresTime;
    protected PresentityManager presentityManager;
    protected Hashtable tagsList;
  
    /** Creates new PresenceServer */
    public PresenceServer() {
        expiresTime=3600;
        presentityManager=new PresentityManager();
        tagsList=new Hashtable();
    }
    
    public int getExpiresTime(){
        return expiresTime;
    }
    
    

    public String getNotifyBody(String presentity,String contactURISender, String status,
        String subStatus) {
        // body:
        String body=

        "<?xml version=\"1.0\"?> \n"+
        "<!DOCTYPE presence \n"+
        "PUBLIC \"-//IETF//DTD RFCxxxx XPIDF 1.0//EN\" \"xpidf.dtd\"> \n"+
        "<presence> \n"+
        "   <presentity uri=\""+presentity+";method=SUBSCRIBE\" /> \n"+
        "   <atom id=\"1009\"> \n"+
        "   <address uri=\"sip:"+contactURISender+";user=ip\" priority=\"0.800000\"> \n"+
        "       <status status=\""+status+"\" /> \n"+
        "       <msnsubstatus substatus=\""+subStatus+"\" /> \n"+
        "   </address> \n"+
        "   </atom> \n"+
        "</presence> \n";
        /*
        "<?xml version=\"1.0\"?> \n"+
        "<!DOCTYPE presence \n"+
        "PUBLIC \"-//IETF//DTD RFCxxxx XPIDF 1.0//EN\" \"xpidf.dtd\"> \n"+
        "<presence> \n"+
        "   <presentity uri=\"sip:rr@ff.v;method=SUBSCRIBE\" /> \n"+
        "   <atom id=\"1003\"> \n"+
        "   <address uri=\"sip:129.6.55.78:2000;user=ip\" priority=\"0.800000\"> \n"+
        "       <status status=\""+status+"\" /> \n"+
        "       <msnsubstatus substatus=\""+subStatus+"\" /> \n"+
        "   </address> \n"+
        "   </atom> \n"+
        "</presence> \n";
*/
        return body;
    }

    
    public void addSubscriber(Request subscribeRequest,ServerTransaction serverTransaction,
    boolean delayed) {
        try{
            String presentity=getKey(subscribeRequest,"To");
            
            String fromURL=getKey(subscribeRequest,"From");
            if ( presentityManager.hasSubscriber(presentity,fromURL) ) {
                ProxyDebug.println("DEBUG, PresenceServer, addSubscriber(), the peer: "+fromURL+" has already"+
                " subscribed for the presentity "+presentity+"!! Let's update the subscription");
                Subscriber subscriber=new Subscriber(fromURL,subscribeRequest,serverTransaction,delayed);
                presentityManager.addSubscriber(presentity,subscriber);
            }
            else {
                Subscriber subscriber=new Subscriber(fromURL,subscribeRequest,serverTransaction,delayed);
                presentityManager.addSubscriber(presentity,subscriber);
                
                //DebugProxy.println("DEBUG, PresenceServer, addSubscriber(), the subscriber"+
                //" "+fromURL+" has been added for the presentity: "+presentity );
            }
        }
        catch(Exception e) {
            ProxyDebug.println("ERROR, PresenceServer, addSubscriber(), Exception raised:");
            e.printStackTrace();
        }
    }
    
    public static URI getCleanUri(URI uri) {
        if (uri instanceof SipURI) {
            SipURI sipURI=(SipURI)uri.clone();
            
            Iterator iterator=sipURI.getParameterNames();
            while (iterator!=null && iterator.hasNext()) {
                String name=(String)iterator.next();
                sipURI.removeParameter(name);
            }
            return  sipURI;
        }
        else return  uri;
    }
    
    
    public  String getKey(Message message,String header) {
        try{
            Address address=null;
            if (header.equals("From") ) {
                FromHeader fromHeader=(FromHeader)message.getHeader(FromHeader.NAME);
                address=fromHeader.getAddress();
                
            }
            else
                if (header.equals("To") ) {
                    ToHeader toHeader=(ToHeader)message.getHeader(ToHeader.NAME);
                    address=toHeader.getAddress();
                    
                }
                
            javax.sip.address.URI  cleanedUri=null;
            if (address==null) {
                cleanedUri= getCleanUri( ((Request)message).getRequestURI());
            }
            else {
                // We have to build the key, all
                // URI parameters MUST be removed:
                cleanedUri = getCleanUri(address.getURI());
            }
            
            if (cleanedUri==null) return null;
            
            String  keyresult=cleanedUri.toString();
            ProxyDebug.println("DEBUG, PresenceServer, getKey(), the key is: " + 
            keyresult);
            return keyresult.toLowerCase();
            
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public boolean hasSubscriberWaiting(Request registerRequest) {
        String fromURL=getKey(registerRequest,"From");
        return presentityManager.hasSubscriberWaiting(fromURL);
    }
    
    
    public Vector getSubscriberList(Request registerRequest) {
        String presentity=getKey(registerRequest,"From");
        return presentityManager.getSubscriberList(presentity);
    }
    
    public void removeSubscriber(Request subscribeRequest) {
        String presentity=getKey(subscribeRequest,"To");
        String fromURL=getKey(subscribeRequest,"From");
        presentityManager.removeSubscriber(presentity,fromURL);
    }
    
    public void removeAllSubscriptions(String fromURL) {
        presentityManager.removeAllSubscriptions(fromURL);
    }
  
    public void createTagsAssociation(String fromTag,String toTag) {
        tagsList.put(fromTag,toTag);
    }
    
   public String getToTag(String fromTag) {
        return  (String)tagsList.get(fromTag);
    }

   
    public void sendSubscribe(Proxy proxy,Request registerRequest) {
        try {
            ProxyDebug.println();
            
            // WE have to check if the REGISTER is not an unregister!!!
            // If yes, we have to remove all the subscriptions of this guy!!!
            ExpiresHeader expiresHeader=(ExpiresHeader)registerRequest.getHeader(ExpiresHeader.NAME);
            if (expiresHeader!=null && expiresHeader.getExpires()==0 ) { 
                String fromURL=getKey(registerRequest,"From");
                ProxyDebug.println("DEBUG, PresenceServer, sendSubscribe(), security:"+
                " we try to remove all the subscriptions of "+fromURL);
                removeAllSubscriptions(fromURL);
            }
            
            if (hasSubscriberWaiting(registerRequest) ) {
                   ProxyDebug.println("DEBUG, PresenceServer, sendSubscribe(), "+
                "  We got some subscribers in waiting... Let's send their SUBSCRIBE...");
                    Vector subscriberList=getSubscriberList(registerRequest);
                    for (int i=0;i<subscriberList.size();i++) {
                        Subscriber subscriber=(Subscriber) subscriberList.elementAt(i);
                        if (subscriber.isSubscribeDelayed() ) {
                            subscriber.setSubscribeDelayed(false);
                            
                            Request subscribeRequest=subscriber.getSubscribeRequest();
                            HeaderFactory headerFactory=proxy.getHeaderFactory();
                            // WE have to add a new Header: "Event"
                            Header header=headerFactory.createHeader("Event","presence");
                            subscribeRequest.setHeader(header);
                            
                            sendSubscribe(proxy,subscribeRequest,subscriber.serverTransaction);
                        }
                    }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    /** If the request has from AND to tag, we retrieve the parent
     * dialog and we get the first transaction, then we check if it
     * is a server or client transaction and we get the mapping from
     * the table, get its dialog and get the tags!
     *
     * If the request has ONLY from tag, we create a new From tag
     * and forward the request!
     **/
    public Request updateTags(Proxy proxy,Request request,ServerTransaction serverTransaction) {
        try{
            ToHeader toHeader=(ToHeader)request.getHeader(ToHeader.NAME);
            FromHeader fromHeader=(FromHeader)request.getHeader(FromHeader.NAME);
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            
            if (fromHeader.getTag()!=null && toHeader.getTag()==null) {
                ProxyDebug.println("PresenceServer, updateTags(), the request "+
                "is the first transaction for the future dialog, we have to create"+
                " a new From tag!" );
                fromHeader.setTag(proxyUtilities.generateTag());
                return request;
            }
            
            TransactionsMapping transactionsMapping =
            proxy.getTransactionsMapping();
            Dialog parentDialog=serverTransaction.getDialog();
            if (parentDialog==null) {
                ProxyDebug.println("PresenceServer, updateTags(), the parent dialog"+
                "is null, we can not update the tags" );
                return request;
            }
            
            Transaction transaction=parentDialog.getFirstTransaction();
            if (transaction==null)  {
                ProxyDebug.println("PresenceServer, updateTags(), the first "+
                "transaction is null, we can not update the tags" );
                return request;
            }
            
            String fromTag=null;
            String toTag=null;
            if (transaction instanceof ServerTransaction) {
                ServerTransaction st=(ServerTransaction)transaction;
                ClientTransaction ct=transactionsMapping.getClientTransaction(st);
                Dialog dialog=ct.getDialog();
                fromTag=dialog.getLocalTag();
                toTag=dialog.getRemoteTag();
                
            }
            else {
                ClientTransaction ct=(ClientTransaction)transaction;
                ServerTransaction st=transactionsMapping.getServerTransaction(ct);
                Dialog dialog=ct.getDialog();
                toTag=dialog.getLocalTag();
                fromTag=dialog.getRemoteTag();
            }
            
            fromHeader.setTag(fromTag);
            toHeader.setTag(toTag);
            
            return request;
            
        }
        catch (Exception ex) {
            ex.printStackTrace();
            return request;
        }
    }
     
     
     public void sendSubscribe(Proxy proxy,Request request,ServerTransaction serverTransaction) {
         try{
             PresenceServer presenceServer=proxy.getPresenceServer();
             ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
             MessageFactory messageFactory=proxy.getMessageFactory();
             Registrar registrar=proxy.getRegistrar();
             HeaderFactory headerFactory=proxy.getHeaderFactory();
             SipProvider sipProvider=proxy.getSipProvider();
             TransactionsMapping transactionsMapping =
             proxy.getTransactionsMapping();
             
             Request requestCloned=(Request)request.clone();
             
             // We have to forward the SUBSCRIBE to the peer that is the target!!!
             // Otherwise, we wait for the guy to register
             // First, we test if we have to deal with an unsubscribe
             ExpiresHeader expiresHeader=(ExpiresHeader)request.getHeader(
             ExpiresHeader.NAME);
             
             if (expiresHeader==null || expiresHeader.getExpires()!=0) {
                 
                 String presentity=presenceServer.getKey(requestCloned,"To");
                 Vector contacts=registrar.getContactHeaders(presentity);
                 if ( registrar.hasRegistration(presentity) && contacts!=null &&
                 !contacts.isEmpty()) {
                     presenceServer.addSubscriber(request,serverTransaction,false);
                     //if (presenceServer.hasSubscribeWaiting(request) ) {
                     ProxyDebug.println("DEBUG, resenceServer, sendSubscribe(), "+
                     " The contact is registered, we forward the SUBSCRIBE");
                     
                     for (int i = 0; i < contacts.size(); i++) {
                         
                         ContactHeader contact = (ContactHeader)
                         contacts.elementAt(i);
                         requestCloned=proxyUtilities.updateRequest(request,
                         (SipURI)contact.getAddress().getURI(),serverTransaction);
                         
                         if (requestCloned!=null) {
                             requestCloned=updateTags(proxy,requestCloned,serverTransaction);
                         }
                         
                         if (requestCloned!=null) {
                             ClientTransaction ct =
                             sipProvider.getNewClientTransaction
                             (requestCloned);
                             sipProvider.sendRequest(ct,requestCloned);
                             
                             ProxyDebug.println
                             ("DEBUG, PresenceServer, sendSubscribe()"+
                             ", request forwarded:\n "
                             +requestCloned);
                             
                             transactionsMapping.addMapping(serverTransaction,ct);
                         }
                         else {
                             ProxyDebug.println("ERROR, resenceServer, sendSubscribe()"+
                             ", PB to forward the request...:\n ");
                         }
                     }
                     
                 }
                 else {
                     // WE have to store the information and wait for a REGISTER
                     ProxyDebug.println("DEBUG: SubscribeProcessing, processRequest(), "+
                     "The receiver of this SUBSCRIBE is not registered, or does "+
                     "not have any contacts: we store the SUBSCRIBE");
                     presenceServer.addSubscriber(request,serverTransaction,true);
                 }
             }
             else {
                 // We have an unsubscribe to process
                 String presentity=presenceServer.getKey(requestCloned,"To");
                 Vector contacts=registrar.getContactHeaders(presentity);
                 
                 // We have to remove the subscription!!!
                 ProxyDebug.println("DEBUG: SubscribeProcessing, processRequest(), "+
                 " Removing a subscription in progress...");
                 presenceServer.removeSubscriber(request);
                 
                 // WE have to send him back a NOTIFY with body (closed:offline)
                    /*
                    NotifyProcessing notifyProcessing=proxy.getNotifyProcessing();
                     
                    String requestURI=presenceServer.getURL(request,"Request-URI");
                    SipStack sipStack=proxy.getSipStack();
                    NistStack nistStack=((SipStackImpl)sipStack).getNistStack();
                     
                    String contactURIsender=((SIPStack)nistStack).getHostAddress()+
                    ":"+((SIPStack)nistStack).getPort();
                    String body=presenceServer.getNotifyBody(requestURI,contactURIsender,
                    "closed","offline");
                    //String body="";
                    //String body=null;
                    notifyProcessing.sendNotify(requestCloned,response,body,"active");
                     
                     */
             }
         }
         catch (Exception ex) {
             ex.printStackTrace();
         }
     }
     

    
}
