/*
 * Registrar.java
 *
 * Created on June 27, 2002, 11:16 AM
 */

package examples.proxy.registrar;

import examples.proxy.*;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.rmi.*;
import java.rmi.server.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;
import java.net.URLEncoder;
import examples.proxy.presenceserver.*;
import examples.proxy.gui.*;
/**
 *
 * @author  deruelle
 * @version 1.0
 */
public class Registrar  extends UnicastRemoteObject implements RegistrarAccess {

    protected  RegistrationsTable registrationsTable;
    protected  RegistrationsList gui;
    protected  Proxy proxy;
   
    // in seconds
    public static long EXPIRES_TIME_MIN=1;
    public static long EXPIRES_TIME_MAX=3600;
    
    protected String xmlRegistrationsFile;
    protected Vector threads;
    /**
     *  Creates new Registrar
     */
    public Registrar(Proxy proxy) throws RemoteException {
	this.proxy = proxy;
        registrationsTable=new RegistrationsTable(this);
    }
    
    public void setRegistrationsList(RegistrationsList registrationsList) {
        this.gui=registrationsList;
    }
    
    public void parseXMLregistrations(String file) {
        try{
            xmlRegistrationsFile=file;
       
            XMLRegistrationsParser xmlRegistrationsParser=new
            XMLRegistrationsParser(xmlRegistrationsFile,proxy);
            Registrations registrations=xmlRegistrationsParser.getRegistrations();
            if (registrations==null) return;
            Vector registrationList=registrations.registrationList;
            if (registrationList!=null) {
	        if (ProxyDebug.debug) {
                  ProxyDebug.println("Uploading of "
		  +registrationList.size()+" registrations");
		}
                for( int i=0;i<registrationList.size();i++) {
                    Registration registration=
				(Registration)registrationList.elementAt(i);
                    registrationsTable.addRegistration(registration);
                }
            }
            
            Vector domainList=registrations.domainList;
            if (domainList!=null) {
                threads=new Vector();
	        if (ProxyDebug.debug) {
                  ProxyDebug.println("Uploading "
		  +domainList.size()+" domains");
		}
                for( int i=0;i<domainList.size();i++) {
                    Domain domain=
				(Domain)domainList.elementAt(i);
                    if (domain.hostName!=null) {
                       RegistrationDomainThread rr=new RegistrationDomainThread(proxy,domain);
                       rr.start();
                       threads.addElement(rr);
                    }
                }
            }
        }
        catch(Exception e) {
	   if (ProxyDebug.debug)  {
                ProxyDebug.println
		("ERROR, Registrar, Registrar(), exception  raised during"+
                " parsing of the static registrations:");
	    }
            e.printStackTrace();
        }
    }
    
    public void clean() {
         if (threads==null) return;
         for( int i=0;i<threads.size();i++) {
                RegistrationDomainThread rr=(RegistrationDomainThread)threads.elementAt(i);
                rr.STOP=true;
         }
    }
    

     public static void writeFile(String outFile, String text) {
        // we read this file to obtain the options
        try{
            FileWriter fileWriter = new FileWriter(outFile,false);
            PrintWriter pw = new PrintWriter(fileWriter,true);
            
            if (text==null) {
                pw.println();
            }
            else
            {
                 pw.println(text);
            }
           
            pw.close();
            fileWriter.close();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

     public void setExpiresTime(long expiresTime) {
        EXPIRES_TIME_MAX=expiresTime;
     }
    
    
    public void writeXMLRegistrations() {
        String registrationsTags=registrationsTable.getXMLTags();
        writeFile(xmlRegistrationsFile,registrationsTags);
    }
    
    
    public RegistrationsTable getRegistrationsTable() {
        return registrationsTable;
    }
    
    
/****************************************************************************/
/*************************** RMI REGISTRY    ********************************/
    
    public String getRegistryXMLTags() throws RemoteException {
        return registrationsTable.getRegistryXMLTags();
    }
    
    

    public synchronized Vector getRegistryBindings() throws RemoteException {
	return registrationsTable.getRegistryBindings();
    }

    public synchronized int getRegistrySize() throws RemoteException {
	 return registrationsTable.getRegistrySize();
    }

    // Need to add more registry query functions here.
    public void initRMIBindings() {
	String name = null;
	try {
          SipStack sipStack=proxy.getSipStack();
	  Iterator it  =sipStack.getListeningPoints();
	  ListeningPoint lp = (ListeningPoint) it.next();
	  String stackIPAddress = sipStack.getIPAddress(); 
	  name = "//" + stackIPAddress + ":" + 0 +  "/" +
              sipStack.getStackName() 
	     + "/" + "test.jainproxy.Registrar";
	   if (ProxyDebug.debug)  {
	      ProxyDebug.println("Exporting Registration Table " + name);
	   }
	   Naming.rebind(name,this);
	} catch (Exception ex) {
            if (ProxyDebug.debug) {
                ProxyDebug.println
                 ("Problem trying to export the Registration Table: " + name);
            }
	    ex.printStackTrace();
	}
    }

/******************************************************************************/
/******************************************************************************/
        
    
    /** Process the register message: add, remove, update the bindings
     *  and manage also the expiration time.
     *  @param Request Register message to set
     *  @return int status code of the process of the Register.
     */
    public
    synchronized void processRegister(Request request,
			ServerTransaction serverTransaction ) {
        try{
            SipProvider sipProvider=proxy.getSipProvider();
            MessageFactory messageFactory=proxy.getMessageFactory();
            
            String key=getKey(request);
            if (key==null){
                if (ProxyDebug.debug) {
                     ProxyDebug.println
                    ("Registrar, processRegister(), key is null"+
                    " 400 INVALID REQUEST replied");
                }
                Response response=messageFactory.createResponse
                    (Response.BAD_REQUEST,request);
                sipProvider.sendResponse(serverTransaction,response);
            }
            else {
                // Add the key if it is a new user:
                if (ProxyDebug.debug){
                    ProxyDebug.println
                    ("Registrar, processRegister(), key: \""+key+"\"");
                }
		// bug report by Alistair Coles
                if ( hasStar(request) && hasExpiresZero(request) ) {
		    if (ProxyDebug.debug) {
                       ProxyDebug.println
		       ("Registrar, processRegister(), (* or expires=0) "+
                        " we remove the registration!!");
		    }
                    registrationsTable.removeRegistration(key);
                    
                    Vector contactHeaders=getContactHeaders(key);
                    Response response=messageFactory.createResponse
			(Response.OK,request);
                    if ( contactHeaders!=null ) {
                        for (int i = 0 ; i < contactHeaders.size(); i++) {
                            ContactHeader contact = (ContactHeader) 
                                    contactHeaders.elementAt(i);
                            response.addHeader(contact);
                        }
                    }
                    if (ProxyDebug.debug) {
                        ProxyDebug.println
                            ("Registrar, processRegister(), response sent:");
                        ProxyDebug.print(response.toString());
                    }
                    sipProvider.sendResponse(serverTransaction,response);
                }
                else {
                    if ( hasContactHeaders(request) ) {
                        if ( registrationsTable.hasRegistration(key) ) {
                            registrationsTable.updateRegistration(key,request);
                        }
                        else {
                            registrationsTable.addRegistration(key,request);
                            
                            // we have to forward SUBSCRIBE if the presence server
                            // is enabled:
                            if (proxy.isPresenceServer()) {
                                PresenceServer presenceServer=proxy.getPresenceServer();
                                presenceServer.sendSubscribe(proxy,request);
                            }
                        }
                        Vector contactHeaders=getContactHeaders(key);
                        Response response=
			  messageFactory.createResponse(Response.OK,request);
                        if ( contactHeaders!=null ) {
                            for (int i = 0; i < contactHeaders.size(); i++) {
                                ContactHeader contact = (ContactHeader)
                                            contactHeaders.elementAt(i);
                                response.addHeader(contact);
                            }
                        }
                        
			if (ProxyDebug.debug)  {
                           ProxyDebug.println
			   ("Registrar, processRegister(), response sent:");
                           ProxyDebug.print(response.toString());
			}
                        sipProvider.sendResponse(serverTransaction,response);
                    }
                    else {
			if (ProxyDebug.debug)  {
                           ProxyDebug.println("Registrar, processRegister(), "+
                            " 400 INVALID REQUEST replied");
			}
                        Response response=
				messageFactory.createResponse
				(Response.BAD_REQUEST,request);
                        sipProvider.sendResponse(serverTransaction,response);
                    }
                }
            }
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    public static URI getCleanUri(URI uri) {
        if (uri instanceof SipURI) {
            SipURI sipURI=(SipURI)uri.clone();
            
            Iterator iterator=sipURI.getParameterNames();
            while (iterator!=null && iterator.hasNext()) {
                String name=(String)iterator.next();
                sipURI.removeParameter(name);
            }
            return  sipURI;
        }
        else return  uri;
    }
    
    /** The key is built following this rule:
     * The registrar extracts the address-of-record from the To header
     * field of the request. The URI
     * MUST then be converted to a canonical form.  To do that, all
     * URI parameters MUST be removed (including the user-param), and
     * any escaped characters MUST be converted to their unescaped
     * form.  The result serves as an index into the list of bindings
     */
    public String getKey(Request request) {
        // Let's see if we already have a binding for this request:
        try{
            ToHeader toHeader=(ToHeader)request.getHeader(ToHeader.NAME);
            Address address=null;
            address  = toHeader.getAddress();
            /*
            if (request.getMethod().equalsIgnoreCase("REGISTER"))
                address  = toHeader.getAddress();
            else
                if (request.getMethod().equalsIgnoreCase("NOTIFY"))
                    address  = toHeader.getAddress();
                else
                    if (request.getMethod().equalsIgnoreCase("INFO"))
                        address  = toHeader.getAddress();
                    else
                        if (request.getMethod().equalsIgnoreCase("MESSAGE"))
                            address  = toHeader.getAddress();
                        else
                            if (request.getMethod().equalsIgnoreCase("SUBSCRIBE"))
                                address  = toHeader.getAddress();
                            else  {}
*/
            javax.sip.address.URI  cleanedUri;
            if (address==null) {
                cleanedUri= getCleanUri(request.getRequestURI());
            }
            else {
                // We have to build the key, all
                // URI parameters MUST be removed:
                cleanedUri = getCleanUri(address.getURI());
            }
            String  keyresult=cleanedUri.toString();
            return keyresult.toLowerCase();
            
        }
        catch(Exception e) {
	    if (ProxyDebug.debug)  {
            		ProxyDebug.println
			("ERROR, Registrar, getKey(), exception raised:");
	    }
            e.printStackTrace();
            return null;
        }
    }
    
    public boolean hasRegistration(String key) {
        return registrationsTable.hasRegistration(key);
    }
    
    public boolean hasDomainRegistered(Request request) {
        try{
            URI uri=request.getRequestURI();
            if (!uri.isSipURI()) return false;
            
            SipURI sipURI=(SipURI)uri;
            String user=sipURI.getUser();
            String host=sipURI.getHost();
           
            return hasRegistration("sip:"+host);
        }
        catch (Exception ex) {
	    if (ProxyDebug.debug)  {
            	ProxyDebug.println
		("Registrar, hasDomainRegistered(), exception raised:"
			+ex.getMessage());
	    }
            ex.printStackTrace();
            return false;
        }
    }
    
    public Vector getDomainContacts(Request request) {
        try{
            URI uri=request.getRequestURI();
            if (!uri.isSipURI()) return null;
            
            SipURI sipURI=(SipURI)uri;
            String user=sipURI.getUser();
            String host=sipURI.getHost(); 
            
            return getContactHeaders("sip:"+host);
        }
        catch (Exception ex) {
	    if (ProxyDebug.debug) {
                ProxyDebug.println
		("Proxy, getDomainContacts(), exception raised:"+ex.getMessage());
	    }
            ex.printStackTrace();
            return null;
        }
    }
     
     
    public boolean hasRegistration(Request request)   {
        try{
            String key = getKey(request);
            return hasRegistration(key);
        }
        catch (Exception ex) {
	    if (ProxyDebug.debug)  {
            	ProxyDebug.println
		("Registrar, hasRegistration(), exception raised:"
			+ex.getMessage());
	    }
            ex.printStackTrace();
            return false;
        }
    }
     
    public Vector getContacts(Request request) {
        try{
            String key=getKey(request);
            return getContactHeaders(key);
        }
        catch (Exception ex) {
	    if (ProxyDebug.debug) {
                ProxyDebug.println
		("Proxy, getContacts(), exception raised:"+ex.getMessage());
	    }
            ex.printStackTrace();
            return null;
        }
    }
     
     
    public boolean hasContactHeaders(Request request) {
         ListIterator list=(ListIterator)request.getHeaders(ContactHeader.NAME);
         return list!=null;
    }
    
    private boolean hasStar(Request request) throws Exception{
        ListIterator list=(ListIterator)request.getHeaders(ContactHeader.NAME);
            
        if (list==null) return false;
        while( list.hasNext() ) {
             ContactHeader contactHeader=(ContactHeader)list.next();
             if (contactHeader.getAddress().isWildcard()  ) {
                 return true;
             }
        }
        return false;
    }
    
    private boolean hasExpiresZero(Request request) {
        try{
            ExpiresHeader expiresHeader=
		(ExpiresHeader)request.getHeader(ExpiresHeader.NAME);
            if (expiresHeader==null) {
                return false;
            }
            else return expiresHeader.getExpires()==0;
        }catch(Exception e){
            e.printStackTrace();
            return false;
        }
    }
    
    
    public Vector getContactHeaders(String key) {
        return registrationsTable.getContactHeaders(key);
    }
    
    public static Vector getContactHeaders(Request request){
        Vector contacts =new Vector();
        try{
            ListIterator list=
		(ListIterator)request.getHeaders(ContactHeader.NAME);
            
            if (list==null) return contacts;
            while( list.hasNext() ) {
                ContactHeader contactHeader=(ContactHeader)list.next();
                contacts.addElement(contactHeader);
            }
            
            // We will sort out the contacts following the "q" parameter
            
            return contacts;
        }
        catch(Exception e){
            e.printStackTrace();
            return contacts;
        }
    }

    protected void printRegistrations(){
        registrationsTable.printRegistrations();
    }
   
}
