/*
 * RegistrationDomainThread.java
 *
 * Created on March 25, 2003, 11:05 AM
 */

package examples.proxy.registrar;

import examples.proxy.*;
import java.io.*;
import javax.sip.*;
import javax.sip.message.*; 
import javax.sip.header.*;
import javax.sip.address.*;
import java.util.*;

/**
 *
 * @author  deruelle
 */
public class RegistrationDomainThread extends Thread{
    
    protected Domain domain;
    protected Proxy proxy;
    protected long callIdCounter;
    protected int cseq;
    protected boolean STOP;
    
    /** Creates a new instance of RegistrationDomainThread */
    public RegistrationDomainThread(Proxy proxy,Domain domain) {
        this.domain=domain;
        this.proxy=proxy;
        callIdCounter=0;
        cseq=0;
        STOP=false;
    }
    
    public void sendDomainRegistration() {
        try {
            ProxyDebug.println();
            ProxyDebug.println("Sending REGISTER (SignIn) in progress for the domain: "+
            domain.hostName);
            
            SipStack sipStack=proxy.getSipStack();
            String stackIPAddress=sipStack.getIPAddress();
            SipProvider sipProvider=proxy.getSipProvider();
            MessageFactory messageFactory=proxy.getMessageFactory();
            HeaderFactory headerFactory=proxy.getHeaderFactory();
            AddressFactory addressFactory=proxy.getAddressFactory();
            ProxyUtilities proxyUtilities=proxy.getProxyUtilities();
            
            String stackDomain=proxy.getStackDomain();
            if (stackDomain==null || stackDomain.trim().equals("") ) return;
            
            // Request-URI:
            SipURI requestURI=addressFactory.createSipURI(null,domain.hostName);
            requestURI.setMAddrParam(domain.hostName);
            if (domain.port!=null)
                requestURI.setPort(Integer.valueOf(domain.port).intValue());
            
            // Call-ID:
            callIdCounter++;
            CallIdHeader callIdHeader=headerFactory.createCallIdHeader(
            "nist-sip-im-register-callId"+callIdCounter);
            
            // CSeq:
            cseq++;
            CSeqHeader cseqHeader=headerFactory.createCSeqHeader(cseq,"REGISTER");
            
            // To header:
            SipURI toURI=addressFactory.createSipURI(null,stackDomain);
            Address toAddress=addressFactory.createAddress(toURI);
            //String toTag=Utils.generateTag();
            ToHeader toHeader=headerFactory.createToHeader(toAddress,null);
            
            // From Header:
            SipURI fromURI=addressFactory.createSipURI(null,stackDomain);
            Address fromAddress=addressFactory.createAddress(fromURI);
            String fromTag=proxyUtilities.generateTag();
            FromHeader fromHeader=headerFactory.createFromHeader(fromAddress,fromTag);
            
            //  Via header
            ViaHeader viaHeader=proxy.getStackViaHeader(); 
            Vector viaList=new Vector();
            viaList.addElement(viaHeader);
            
            // MaxForwards header:
            MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(10);
            
            
            Request request=messageFactory.createRequest(requestURI,"REGISTER",
            callIdHeader,cseqHeader,fromHeader,toHeader,viaList,maxForwardsHeader);
            
            ClientTransaction clientTransaction=sipProvider.getNewClientTransaction(request);
            if (clientTransaction==null) {
                ProxyDebug.println("ERROR, the client transaction is null for the"+
                " request"+request);    
                return;
            }
            
            // Contact header:
            ContactHeader contactHeader=proxy.getStackContactHeader(); 
            request.setHeader(contactHeader);
           
            sipProvider.sendRequest(clientTransaction,request);
            ProxyDebug.println("REGISTER sent:\n"+request);
            ProxyDebug.println();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
     /** 
     */    
    public void  run() {  
        ProxyDebug.println("Proxy sending REGISTER............. ");
        try {
           while (!STOP) {
                sendDomainRegistration();
                Thread.sleep(60000);
           }
          
        }
        catch(Exception e) {
            
        }
    }
    
    
}
