/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/
package gov.nist.javax.sip.header;

import gov.nist.core.*;
import javax.sip.*;
import javax.sip.header.*;
import java.text.ParseException;

/**
* Event SIP Header.
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*@author Olivier Deruelle <deruelle@nist.gov><br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/
public class Event extends ParametersHeader implements EventHeader{
    
    protected String eventType;
    
    /** Creates a new instance of Event */
    public Event() {
        super(EVENT);
    }
    
      /**
     * Sets the eventType to the newly supplied eventType string.
     *
     * @param eventType - the  new string defining the eventType supported
     * in this EventHeader
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the eventType value.
     */
    public void setEventType(String eventType) throws ParseException {
       if (eventType==null)  throw new  NullPointerException("JAIN-SIP Exception, Event,"+
       " setEventType(), the eventType parameter is null");
       this.eventType=eventType;
    }

    /**
     * Gets the eventType of the EventHeader. 
     *
     * @return the string object identifing the eventType of EventHeader.
     */
    public String getEventType() {
        return eventType;
    }
    
    /**
     * Sets the id to the newly supplied <var>eventId</var> string.
     *
     * @param eventId - the new string defining the eventId of this EventHeader
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the eventId value.
     */
    public void setEventId(String eventId) throws ParseException {
        if (eventId==null)  throw new  NullPointerException("JAIN-SIP Exception, Event,"+
       " setEventId(), the eventId parameter is null");
        setParameter(ParameterNames.ID,eventId); 
    }

    /**
     * Gets the id of the EventHeader. This method may return null if the 
     * "eventId" is not set.
     * @return the string object identifing the eventId of EventHeader.
     */
    public String getEventId() {
        return getParameter(ParameterNames.ID);
    }
    
    /**
     * Encode in canonical form.
     * @return String
     */
    public String encodeBody() {
            StringBuffer retval = new StringBuffer();
           
            if (eventType!=null) retval.append(SP + eventType + SP);
            
            if (!parameters.isEmpty()) retval.append(SEMICOLON + 
                        this.parameters.encode());
            return retval.toString();
    }
    
}
