

package gov.nist.javax.sip.parser;
import java.util.Hashtable;
import java.lang.reflect.*;
import javax.sip.header.*;
import java.text.ParseException;
import gov.nist.core.*;

/** A factory class that does a name lookup on a registered parser and
* returns a header parser for the given name.
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public class ParserFactory {

    private static Hashtable parserTable; 
    private static final String PARSER_PACKAGE = "gov.nist.javax.sip.parser.";
    private static Class[] constructorArgs;
    
    static {
        parserTable  = new Hashtable();
        constructorArgs = new Class[1];
        constructorArgs[0] = new String().getClass();
        
        try {
            parserTable.put
            (ReplyToHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE + "ReplyToParser"));

            parserTable.put
            (InReplyToHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE + "InReplyToParser"));

            parserTable.put
            (AcceptEncodingHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE + "AcceptEncodingParser"));

            parserTable.put
            (AcceptLanguageHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE + "AcceptLanguageParser"));

	    
            parserTable.put
            ("t",Class.forName(PARSER_PACKAGE+"ToParser"));
            parserTable.put
            (ToHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE+"ToParser"));
            
            
            parserTable.put
            (FromHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE+"FromParser"));
            parserTable.put
            ("f",Class.forName(PARSER_PACKAGE+"FromParser"));
            
            parserTable.put
            (CSeqHeader.NAME.toLowerCase()
		,Class.forName(PARSER_PACKAGE+"CSeqParser"));
            
            parserTable.put
            (ViaHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE+"ViaParser"));
            parserTable.put
            	("v",Class.forName(PARSER_PACKAGE+"ViaParser"));
            
            parserTable.put
            (ContactHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ContactParser"));
            parserTable.put
            ("m",Class.forName(PARSER_PACKAGE+"ContactParser"));
            
            parserTable.put
            (ContentTypeHeader.NAME.toLowerCase(),Class.forName
            (PARSER_PACKAGE + "ContentTypeParser"));
            
            
            parserTable.put
            (ContentLengthHeader.NAME.toLowerCase(),Class.forName
            (PARSER_PACKAGE+"ContentLengthParser"));
            parserTable.put
            ("l",Class.forName(PARSER_PACKAGE+"ContentLengthParser"));
            
            
            parserTable.put
            (AuthorizationHeader.NAME.toLowerCase(),
		Class.forName(PARSER_PACKAGE+
            		"AuthorizationParser"));
            
            parserTable.put
            (WWWAuthenticateHeader.NAME.toLowerCase(),
			Class.forName(PARSER_PACKAGE+
            		"WWWAuthenticateParser"));
            
            parserTable.put
            (CallIdHeader.NAME.toLowerCase(),
			Class.forName(PARSER_PACKAGE +
            		"CallIDParser"));
            parserTable.put
            ("i",Class.forName(PARSER_PACKAGE + "CallIDParser"));
            
            parserTable.put(
            RouteHeader.NAME.toLowerCase(),Class.forName(PARSER_PACKAGE +
            "RouteParser"));
            
            parserTable.put(
            RecordRouteHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE +
            "RecordRouteParser"));
            
            parserTable.put(
            DateHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE + "DateParser"));

            parserTable.put(
            ProxyAuthorizationHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ProxyAuthorizationParser"));
            
            parserTable.put(
            ProxyAuthenticateHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ProxyAuthenticateParser"));

            parserTable.put(
            RetryAfterHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"RetryAfterParser"));

            parserTable.put(
            RequireHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"RequireParser"));

            parserTable.put(
            ProxyRequireHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ProxyRequireParser"));

            parserTable.put( TimeStampHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"TimeStampParser"));

            parserTable.put( UnsupportedHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"UnsupportedParser"));

            parserTable.put( UserAgentHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"UserAgentParser"));

            parserTable.put( SupportedHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"SupportedParser"));

            parserTable.put( ServerHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ServerParser"));

            parserTable.put( SubjectHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"SubjectParser"));

            parserTable.put( SubscriptionStateHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"SubscriptionStateParser"));

            parserTable.put( MaxForwardsHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"MaxForwardsParser"));

            parserTable.put( MimeVersionHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"MimeVersionParser"));

            parserTable.put( MinExpiresHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"MinExpiresParser"));

            parserTable.put( OrganizationHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"OrganizationParser"));

            parserTable.put( PriorityHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"PriorityParser"));

            parserTable.put( RAckHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"RAckParser"));

            parserTable.put( RSeqHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"RSeqParser"));

            parserTable.put( ReasonHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ReasonParser"));

            parserTable.put( WarningHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"WarningParser"));
            
            parserTable.put
            (ExpiresHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ExpiresParser"));
            
            parserTable.put
            (EventHeader.NAME.toLowerCase(),
			Class.forName(PARSER_PACKAGE +
            		"EventParser"));
            parserTable.put
            ("o",Class.forName(PARSER_PACKAGE + "EventParser"));
            
            parserTable.put
            (ErrorInfoHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ErrorInfoParser"));
            
            parserTable.put
            (ContentLanguageHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ContentLanguageParser"));
             
            parserTable.put
            (ContentEncodingHeader.NAME.toLowerCase(),
			Class.forName(PARSER_PACKAGE +
            		"ContentEncodingParser"));
            parserTable.put
            ("e",Class.forName(PARSER_PACKAGE + "ContentEncodingParser"));
            
            parserTable.put
            (ContentDispositionHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"ContentDispositionParser"));
            
            parserTable.put
            (CallInfoHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"CallInfoParser"));
            
            parserTable.put
            (AuthenticationInfoHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"AuthenticationInfoParser"));
            
            parserTable.put
            (AllowHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"AllowParser"));
            
            parserTable.put
            (AllowEventsHeader.NAME.toLowerCase(),
			Class.forName(PARSER_PACKAGE +
            		"AllowEventsParser"));
            parserTable.put
            ("u",Class.forName(PARSER_PACKAGE + "AllowEventsParser"));
            
            parserTable.put
            (AlertInfoHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"AlertInfoParser"));
            
            parserTable.put
            (AcceptHeader.NAME.toLowerCase(),
            Class.forName(PARSER_PACKAGE+"AcceptParser"));

        } catch (ClassNotFoundException ex) {
		// An internal error happened.
            ex.printStackTrace();
            System.exit(0);
        }
    }
    /** create a parser for a header. This is the parser factory.
     */
    public static HeaderParser createParser(String line) throws ParseException {
            String headerName=Lexer.getHeaderName(line);
            String headerValue=Lexer.getHeaderValue(line);
            if (headerName==null || headerValue ==null)
                throw new ParseException
                ("The header name or value is null",0);
            
            Class parserClass =
            (Class) parserTable.get(headerName.toLowerCase());
            
            if (parserClass != null) {
                try {
                    
                    Constructor cons = parserClass.getConstructor
                    (constructorArgs);
                    Object[] args = new Object[1];
                    args[0] = line;
                    HeaderParser retval = (HeaderParser) cons.newInstance(args);
                    return retval;
                    
                } catch (Exception ex) {
		    InternalErrorHandler.handleException(ex);
		    return null; // to placate the compiler.
                }
                       
            } else {
                // Just generate a generic SIPHeader. We define
                // parsers only for the above.
                return new HeaderParser(line);
            }
            
    }


}
