/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/

package gov.nist.javax.sip.stack;

/**
* This class stores a message along with some other informations
* Used to log messages.
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*@author Marc Bednarek  <br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

class MessageLog {
 
    private String message;

    private String source;

    private String destination;

    private long timeStamp;

    private boolean isSender;

    private String cseq;

    private String firstLine;

    private String statusMessage;
	
    private String tid;


    /**
     * Constructor
     */
 
    public MessageLog(String message, String source, String destination,
		      String timeStamp, boolean isSender, String cseq, 
		      String firstLine, String statusMessage, 
		      String tid) {
	if (message == null
	  || message.equals("")) 
		throw new IllegalArgumentException("null msg");
        this.message = message;
        this.source = source;
        this.destination = destination;
	try {
	   long ts = Long.parseLong(timeStamp);
	   if (ts < 0) 
		throw new IllegalArgumentException("Bad time stamp ");
           this.timeStamp = ts;
	} catch (NumberFormatException ex) {
	   throw new IllegalArgumentException("Bad number format " + timeStamp);
	}
        this.isSender = isSender;
        this.cseq = cseq;
	this.firstLine = firstLine;
	this.statusMessage = statusMessage;
	this.tid = tid;
    }

    protected long getTimeStamp() { 
	return this.timeStamp;
    }

    public MessageLog(String message, String source, String destination,
		      long timeStamp, boolean isSender, String cseq, 
		      String firstLine, String statusMessage, 
		      String tid) {
	if (message == null
	  || message.equals("")) 
		throw new IllegalArgumentException("null msg");
        this.message = message;
        this.source = source;
        this.destination = destination;
	if (timeStamp < 0) throw new IllegalArgumentException("negative ts");
	this.timeStamp = timeStamp ;
        this.isSender = isSender;
        this.cseq = cseq;
	this.firstLine = firstLine;
	this.statusMessage = statusMessage;
	this.tid = tid;
    }

    /**
     * Get an XML String for this message
     */
 
    public String flush(long startTime) {
	String log;

	if (statusMessage != null) {
		log = "<message from=\"" + source + 
	    	"\" to=\"" + destination + 
	    	"\" time=\"" + (timeStamp - startTime) + 
	    	"\" statusMessage=\"" + statusMessage +
	    	"\" transactionId=\"" + tid +
	    	"\" firstLine=\"" + firstLine + 
	    	"\">\n";
		log += "<![CDATA[";	
		log += message;
		log += "]]>\n";
		log += "</message>\n";	
	} else {
		log = "<message from=\"" + source + 
	    	"\" to=\"" + destination + 
	    	"\" time=\"" + (timeStamp - startTime) + 
	    	"\" transactionId=\"" + tid +
	    	"\" firstLine=\"" + firstLine + 
	    	"\">\n";
		log += "<![CDATA[";	
		log += message;
		log += "]]>\n";
		log += "</message>\n";	
	}
	return log;
    }
    
}
