/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/

package gov.nist.javax.sip.stack;

import java.util.*;
import java.io.*;

/**
*This class stores a sorted list messages for logging. 
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*@author Marc Bednarek 
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public class MessageLogList extends TreeSet {
    private PrintWriter printWriter;
    private String logFileName;
    private static boolean logToFile = false;
    private String description;
    protected static long startTime;

	

    static {
        startTime = -1;
    }

    /** Constructor.
    *@param comp comparator for sorting the logs
    */
    public MessageLogList(Comparator comp) {
	super(comp);
    }

    /** set a descriptive string for this log (for id purposes).
     *@param description is the decriptive string to add.
     */
    public void addDescription(String description) {
	this.description = description;
    }

    private void checkLogFile()  throws IOException{
		
		if (printWriter != null) return;
		if (logFileName == null) return;
		try {
			File  logFile = new File(logFileName);
			if (! logFile.exists()) {
				logFile.createNewFile();
				printWriter = null;
			}
			// Append buffer to the end of the file.
			if (printWriter == null) {
	   			FileWriter fw = 
					new FileWriter(logFileName,true);
				printWriter = new PrintWriter(fw,true);
			}
		} catch (IOException ex) {
			ex.printStackTrace();
			throw ex;
		}
	}

    /** Constructor given callId and a comparator.
    *@param callId is the call id for which to store the log.
    *@param comp is the comparator to sort the log records.
    */

    public MessageLogList(String callId, Comparator comp) {
	super(comp);
	if (logToFile) {
           logFileName = "trace"+callId+".xml";
	   try {
  	     checkLogFile();
	   } catch (IOException ex) {
		return;
	   }
	   printWriter.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	   printWriter.println("<messages>");
	}
    }

    /** Add a comparable object to the messgageLog 
    *@param obj is the comparable object to add to the message log.
    */
    public synchronized boolean add(Object obj) {
	MessageLog log = (MessageLog) obj;
	if (log.getTimeStamp() < startTime ||
		startTime < 0 ) startTime = log.getTimeStamp();
	return super.add(obj);
    }
	

    /**
     * Get an XML String for this message list. 
     * If the log is being stored on a file,
     * commit the log to the file.
     *@return a string containing the enitre log that we have gathered. 
     * This is sorted in an order that is dictated by the comparator.
     */
 
    public synchronized String flush() {
	StringBuffer log = new StringBuffer();

	// Header
	log.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	if (this.description != null) {
	  log.append("<messages\n").
	  append("  logDescription= \"" + this.description +"\"\n").
	  append(">\n");
	} else {
	  log.append("<messages>\n");
	}

	// Body
	Iterator li = this.iterator();
	while (li.hasNext()) {
	    log.append(((MessageLog)li.next()).flush(startTime));
	}

	// Footer
	log.append("</messages>\n");
	if (logToFile && printWriter != null) {
	    printWriter.println("</messages>");
	}
	// System.out.println("log = " + log.toString());
	return log.toString();
    }

}
