package tools.sniffer;

import java.util.*;
import gov.nist.javax.sip.parser.*;
/** 
* A list of Sniff Sessions.
* Acknowledgement:
* This code was contributed by Tim Bardzil <bardzil@colorado.edu>.
* This code was completed as part of a class project in TLEN 5843
* Singaling Protocols, taught by Professor Douglas C. Sicker, Ph.D. at
* the University of Colorado, Boulder.
* Minor modifications to the code were made by M. Ranganathan <mranga@nist.gov>.
*
*@author Tim Bardzil <bardzil@colorado.edu>
*
*/

public class SniffSessionList extends ArrayList {

  /** 
   * Add a new SniffMessage to the SniffSessionList.
   * Create a new Sniff Session if this is a new call id.
   */
  public void add(SniffMessage sniffMessage) {
    boolean newSession = true;
    ListIterator i = super.listIterator();
    while (i.hasNext()) {
      SniffMessageList temp = (SniffMessageList)i.next();
      if (temp.getCallID().equals(sniffMessage.getCallID())) {
        temp.add(sniffMessage);
        newSession = false;
      }
    }
    if (newSession == true) {
      SniffMessageList newMessageList = new SniffMessageList();
      newMessageList.add(sniffMessage);
      super.add(newMessageList);
    }
  }

  /** 
  * Return a string consisting of formatted messages that can be fed
  * to the trace viewer.
  */
   public String[]  toXML() {
	String[] retval = new String[this.size()];
	ListIterator li = super.listIterator();
	int i = 0;
	while (li.hasNext()) {
           SniffMessageList sml = (SniffMessageList)li.next();
	   retval[i++] = sml.toXML();
	}
	return retval;
   }

   /** 
   * Return an array of call identifiers for the traces.
   */
   public String[] getCallIds() {
	ListIterator li = super.listIterator();
	String[] retval = new String[this.size()];
	int i = 0;
	while (li.hasNext()) {
		SniffMessageList temp = (SniffMessageList) li.next();
		retval[i++] = temp.getCallID();
	}
	return retval;
   }
}
